@@ -13,6 +13,8 @@ from honeybee.aperture import Aperture
 from honeybee.door import Door
 from honeybee.boundarycondition import Surface
 
+from honeybee_radiance_command.options.rfluxmtx import RfluxmtxControlParameters
+
 import os
 
 
@@ -140,7 +142,7 @@ class DynamicShadeGroup(object):
                     states.append(obj.properties.radiance._states[-1])
                 except IndexError:  # no states assigned; create default a dummy state
                     st = RadianceShadeState()
-                    st._parent  = obj
+                    st._parent = obj
                     states.append(st)
         return states
 
@@ -206,7 +208,7 @@ class DynamicShadeGroup(object):
         mod_dup = modifier.duplicate()  # duplicate to avoid editing the original
         # the hidden _bsdf_file property is edited since the file has not yet been copied
         mod_dup._bsdf_file = os.path.join('model', 'bsdf', bsdf_name)
-        mod_strs.append(mod_dup.to_radiance(minimal))
+        mod_strs.insert(1, mod_dup.to_radiance(minimal))
 
     @staticmethod
     def _instance_in_array(object_instance, object_array):
@@ -246,6 +248,7 @@ class DynamicSubFaceGroup(DynamicShadeGroup):
         * is_opaque
         * is_indoor
         * states_json_list
+
     """
     __slots__ = ()
 
@@ -289,6 +292,52 @@ class DynamicSubFaceGroup(DynamicShadeGroup):
                 })
         return states_list
 
+    def rfluxmtx_control_params(self, state_index=0, sampling=None, up_direction=None):
+        """Create a formatted Rfluxmtx control parameters string.
+
+        The optional values are sampling and up_direction. If these values are not
+        provided they will be assigned based on the first State of the dynamic object.
+
+        The control params are only meaningful for BSDF modifiers with one of the
+        sampling types listed below.
+
+        Args:
+            sampling: Set hemisphere sampling type. Acceptable inputs for hemisphere
+                sampling type are:
+
+                    * u for uniform.(Usually applicable for ground).
+                    * kf for klems full.
+                    * kh for klems half.
+                    * kq for klems quarter.
+                    * rN for Reinhart - Tregenza type skies. N stands for subdivisions
+                        and defaults to 1.
+                    * scN for shirley-chiu subdivisions.
+
+                Add a ``-`` in front of the input for left-handed coordinates. For more
+                information see rfluxmtx docs.
+                https://www.radiance-online.org/learning/documentation/manual-pages/pdfs/rfluxmtx.pdf/at_download/file
+            up_direction: Orient the "up" direction for the hemisphere using the
+                indicated axis or a direction vector. Valid inputs include the
+                following. [-]{X|Y|Z|ux,uy,uz}. (Default: Y).
+
+        Returns:
+            RfluxmtxControlParameters
+
+        """
+        state = self.states_by_index(state_index)[0]
+        if up_direction is None:
+            up_direction = state.vmtx_geometry.plane.y
+        if sampling is None:
+            sampling = state.modifier.sampling_type
+            if sampling is None:
+                raise ValueError(
+                    'Rfluxmtx control parameters can only be generated for states '
+                    'with BSDF modifier. Current modifier: %s is a %s.' % (
+                        state.modifier, self.modifier.__class__.__name__)
+                )
+        up_direction = tuple(up_direction)
+        return RfluxmtxControlParameters(sampling, up_direction)
+
     def states_by_index(self, state_index):
         """Get an array of state objects representing a single state for this group.
 
@@ -313,7 +362,7 @@ class DynamicSubFaceGroup(DynamicShadeGroup):
                     states.append(obj.properties.radiance._states[-1])
                 except IndexError:  # no states assigned; create default a dummy state
                     st = RadianceSubFaceState()
-                    st._parent  = obj
+                    st._parent = obj
                     states.append(st)
         return states
 
@@ -343,7 +392,7 @@ class DynamicSubFaceGroup(DynamicShadeGroup):
             if isinstance(mod, BSDF):
                 self._process_bsdf_modifier(mod, blk_str, minimal)
             else:
-                blk_str.append(mod.to_radiance(minimal))
+                blk_str.insert(1, mod.to_radiance(minimal))
         return '\n\n'.join(blk_str)
 
     def tmxt_bsdf(self, state_index):
@@ -381,11 +430,19 @@ class DynamicSubFaceGroup(DynamicShadeGroup):
                 in a minimal format (with spaces instead of line breaks). Default: False.
         """
         states = self.states_by_index(state_index)
+        # create unique glow modifier for aperture group
+        unique_glow = white_glow.duplicate()
+        unique_glow.identifier = 'white_glow_{}'.format(self.identifier)
         # get rad strings for the white_glow modifier and geometry.
         state_str = ['# VMTX representation for "{}"'.format(self.identifier),
-                     white_glow.to_radiance(minimal)]
+                     unique_glow.to_radiance(minimal)]
+
+        # use first state to add the header
+        header = self.rfluxmtx_control_params(state_index=state_index)
+        state_str.append(header.to_radiance())
+
         for state in states:
-            state_str.append(state.vmtx_to_radiance(minimal))
+            state_str.append(state.vmtx_to_radiance(unique_glow, minimal))
         return '\n\n'.join(state_str)
 
     def dmtx_to_radiance(self, state_index, minimal=False):
@@ -403,6 +460,11 @@ class DynamicSubFaceGroup(DynamicShadeGroup):
         # get rad strings for the white_glow modifier and geometry.
         state_str = ['# DMTX representation for "{}"'.format(self.identifier),
                      white_glow.to_radiance(minimal)]
+
+        # use first state to add the header
+        header = self.rfluxmtx_control_params()
+        state_str.append(header.to_radiance())
+
         for state in states:
             state_str.append(state.dmtx_to_radiance(minimal))
         return '\n\n'.join(state_str)
