@@ -4,4 +4,4 @@ from .cie import CIE
 from .climatebased import ClimateBased
 from .sunmatrix import SunMatrix
 from .skymatrix import SkyMatrix
-from .skydome import SkyDome
+from .skydome import SkyDome, UniformSky
