@@ -47,6 +47,13 @@ class Trans(Material):
         * roughness
         * transmitted_diff
         * transmitted_spec
+        * average_reflectance
+        * average_absorption
+        * average_transmittance
+        * diffuse_reflectance
+        * diffuse_transmittance
+        * specular_transmittance
+        * specular_sampling_threshold
         * values
         * modifier
         * dependencies
@@ -56,9 +63,11 @@ class Trans(Material):
     __slots__ = ('_r_reflectance', '_g_reflectance', '_b_reflectance',
                  '_specularity', '_roughness', '_transmitted_diff', '_transmitted_spec')
 
-    def __init__(self, identifier, r_reflectance=0.0, g_reflectance=0.0, b_reflectance=0.0,
-                 specularity=0.0, roughness=0.0, transmitted_diff=0.0,
-                 transmitted_spec=0.0, modifier=None, dependencies=None):
+    def __init__(
+        self, identifier, r_reflectance=0.0, g_reflectance=0.0, b_reflectance=0.0,
+        specularity=0.0, roughness=0.0, transmitted_diff=0.0,
+        transmitted_spec=0.0, modifier=None, dependencies=None
+    ):
         """Create trans material."""
         Material.__init__(self, identifier, modifier=modifier,
                           dependencies=dependencies)
@@ -81,7 +90,7 @@ class Trans(Material):
 
     @property
     def r_reflectance(self):
-        """Reflectance for red channel.
+        """Get or set the reflectance for red channel.
 
         The value should be between 0 and 1 (Default: 0).
         """
@@ -94,7 +103,7 @@ class Trans(Material):
 
     @property
     def g_reflectance(self):
-        """Reflectance for green channel.
+        """Get or set the reflectance for green channel.
 
         The value should be between 0 and 1 (Default: 0).
         """
@@ -107,7 +116,7 @@ class Trans(Material):
 
     @property
     def b_reflectance(self):
-        """Reflectance for blue channel.
+        """Get or set the reflectance for blue channel.
 
         The value should be between 0 and 1 (Default: 0).
         """
@@ -120,7 +129,7 @@ class Trans(Material):
 
     @property
     def specularity(self):
-        """Fraction of specularity.
+        """Get or set the fraction of specularity.
 
         In most cases specularity fractions greater than 0.1 are not common in
         non-metallic materials (Default: 0).
@@ -133,7 +142,7 @@ class Trans(Material):
 
     @property
     def roughness(self):
-        """Roughness is specified as the rms slope of surface facets.
+        """Get or set the roughness as the rms slope of surface facets.
 
         A value of 0 corresponds to a perfectly smooth surface, and a value of 1
         would be a very rough surface. Roughness values greater than 0.2 are not
@@ -147,7 +156,7 @@ class Trans(Material):
 
     @property
     def transmitted_diff(self):
-        """Transmitted diffuse.
+        """Get or set the transmitted diffuse.
 
         The transmitted diffuse component is the fraction of transmitted light that is
         transmitted diffusely in as scattering fashion.
@@ -161,7 +170,7 @@ class Trans(Material):
 
     @property
     def transmitted_spec(self):
-        """Transmitted specular.
+        """Get or set the transmitted specular.
 
         The transmitted specular component is the fraction of transmitted light that is
         not diffusely scattered.
@@ -175,13 +184,45 @@ class Trans(Material):
 
     @property
     def average_reflectance(self):
-        """Calculate average reflectance of trans material."""
-        return (0.265 * self.r_reflectance + 0.670 * self.g_reflectance +
-                0.065 * self.b_reflectance) * (1 - self.specularity) + self.specularity
+        """Get the average reflectance of over the RGB values of the material."""
+        col_ref = 0.265 * self.r_reflectance + 0.670 * self.g_reflectance + \
+            0.065 * self.b_reflectance
+        dif_ref = (1 - self._specularity) * col_ref * (1 - self._transmitted_diff)
+        return self._specularity + dif_ref
+
+    @property
+    def average_absorption(self):
+        """Get the average absorption of over the RGB values of the material."""
+        return (1 - (0.265 * self.r_reflectance + 0.670 * self.g_reflectance +
+                0.065 * self.b_reflectance)) * (1 - self._specularity)
+
+    @property
+    def average_transmittance(self):
+        """Get the total transmittance over the material."""
+        return (1 - self._specularity) * self._transmitted_diff * \
+            (0.265 * self.r_reflectance + 0.670 * self.g_reflectance + 0.065 *
+             self.b_reflectance)
+
+    @property
+    def diffuse_reflectance(self):
+        """Get the average diffuse reflectance of over the RGB values of the material."""
+        col_ref = 0.265 * self.r_reflectance + 0.670 * self.g_reflectance + \
+            0.065 * self.b_reflectance
+        return (1 - self._specularity) * col_ref * (1 - self._transmitted_diff)
+
+    @property
+    def diffuse_transmittance(self):
+        """Get the total transmitted diffuse component."""
+        return self.average_transmittance * (1 - self._transmitted_spec)
+
+    @property
+    def specular_transmittance(self):
+        """Get the total transmitted specular component."""
+        return self.average_transmittance * self._transmitted_spec
 
     @property
     def specular_sampling_threshold(self):
-        """Suggested specular sampling threshold (-st)."""
+        """Get the suggested specular sampling threshold (-st)."""
         return self.transmitted_diff * self.transmitted_spec * \
             (1 - (0.265 * self.r_reflectance + 0.670 * self.g_reflectance +
                   0.065 * self.b_reflectance)) * self.specularity
@@ -198,8 +239,8 @@ class Trans(Material):
 
         Args:
             identifier: Text string for a unique Material ID. Must not contain spaces
-            or special characters. This will be used to identify the object across
-            a model and in the exported Radiance files.
+                or special characters. This will be used to identify the object across
+                a model and in the exported Radiance files.
             r_reflectance: Reflectance for red. The value should be between 0 and 1
                 (Default: 0).
             g_reflectance: Reflectance for green. The value should be between 0 and 1
@@ -263,8 +304,8 @@ class Trans(Material):
     @classmethod
     def from_single_reflectance(
         cls, identifier, rgb_reflectance=0.0, specularity=0.0, roughness=0.0,
-        transmitted_diff=0.0, transmitted_spec=0.0, modifier=None,
-            dependencies=None):
+        transmitted_diff=0.0, transmitted_spec=0.0, modifier=None, dependencies=None
+    ):
         """Create trans material with single reflectance value.
 
         Args:
@@ -288,11 +329,70 @@ class Trans(Material):
                 argument is only useful for defining advanced primitives where the
                 primitive is defined based on other primitives. (Default: [])
         """
-        return cls(identifier, r_reflectance=rgb_reflectance, g_reflectance=rgb_reflectance,
-                   b_reflectance=rgb_reflectance, specularity=specularity,
-                   roughness=roughness, transmitted_diff=transmitted_diff,
-                   transmitted_spec=transmitted_spec, modifier=modifier,
-                   dependencies=dependencies)
+        return cls(
+            identifier, r_reflectance=rgb_reflectance, g_reflectance=rgb_reflectance,
+            b_reflectance=rgb_reflectance, specularity=specularity, roughness=roughness,
+            transmitted_diff=transmitted_diff, transmitted_spec=transmitted_spec,
+            modifier=modifier, dependencies=dependencies)
+
+    @classmethod
+    def from_average_properties(
+        cls, identifier, average_reflectance=0.0, average_transmittance=0.0,
+        is_specular=False, is_diffusing=True, roughness=0.0,
+        modifier=None, dependencies=None
+    ):
+        """Create trans material from average reflectance and transmittance.
+
+        The sum of average_reflectance and average_transmittance must be less than
+        one and any energy not transmitted or reflected is assumed to be absorbed.
+        The resulting material will always be grey with equivalent red, green and
+        blue channels.
+
+        Args:
+            identifier: Text string for a unique Material ID. Must not contain spaces
+                or special characters. This will be used to identify the object across
+                a model and in the exported Radiance files.
+            average_reflectance: The average reflectance of the material. The value
+                should be between 0 and 1 (Default: 0).
+            average_transmittance: The average transmittance of the material. The value
+                should be between 0 and 1 (Default: 0).
+            is_specular: Boolean to note if the reflected component is specular (True)
+                or diffuse (False). (Default: False).
+            is_diffusing: Boolean to note if the tranmitted component is diffused (True)
+                instead of specular like glass (False). (Default: True).
+            roughness: Roughness is specified as the rms slope of surface facets. A value
+                of 0 corresponds to a perfectly smooth surface, and a value of 1 would be
+                a very rough surface. Roughness values greater than 0.2 are not very
+                realistic. (Default: 0).
+            modifier: Material modifier (Default: None).
+            dependencies: A list of primitives that this primitive depends on. This
+                argument is only useful for defining advanced primitives where the
+                primitive is defined based on other primitives. (Default: [])
+        """
+        # check to be sure the input values are valid
+        absorb = 1 - average_reflectance - average_transmittance
+        assert absorb >= 0, 'Sum of average_reflectance and average_transmittance ' \
+            'must be less than or equal to one. Got {}.'.format(
+                average_reflectance + average_transmittance)
+        # determine the reflectance values
+        if is_specular:
+            spec_ref = average_reflectance
+            diff_ref = average_transmittance / (1 - average_reflectance)
+            total_trans = 1
+        else:
+            spec_ref, total_ref = 0, 1 - absorb
+            diff_ref = total_ref
+            total_trans = average_transmittance / total_ref if total_ref != 0 else 0
+        # determine the transmittance values
+        if is_diffusing:
+            diff_trans, spec_trans = total_trans, 0
+        else:
+            diff_trans, spec_trans = total_trans, 1
+        # create the modifier
+        return cls.from_single_reflectance(
+            identifier, rgb_reflectance=diff_ref, specularity=spec_ref,
+            transmitted_diff=diff_trans, transmitted_spec=spec_trans,
+            roughness=roughness, modifier=modifier, dependencies=dependencies)
 
     @classmethod
     def from_primitive_dict(cls, primitive_dict):
@@ -312,15 +412,8 @@ class Trans(Material):
             "dependencies": []
             }
         """
-        assert 'type' in primitive_dict, 'Input dictionary is missing "type".'
-        if primitive_dict['type'] != cls.__name__.lower():
-            raise ValueError(
-                'Type must be %s not %s.' % (
-                    cls.__name__.lower(), primitive_dict['type'])
-            )
-
+        cls._dict_type_check(cls.__name__, primitive_dict)
         modifier, dependencies = cls.filter_dict_input(primitive_dict)
-
         values = primitive_dict['values'][2]
 
         cls_ = cls(
@@ -334,7 +427,8 @@ class Trans(Material):
             transmitted_spec=values[6],
             modifier=modifier,
             dependencies=dependencies)
-        if 'display_name' in primitive_dict and primitive_dict['display_name'] is not None:
+        if 'display_name' in primitive_dict and \
+                primitive_dict['display_name'] is not None:
             cls_.display_name = primitive_dict['display_name']
 
         # this might look redundant but it is NOT. see glass for explanation.
@@ -351,9 +445,9 @@ class Trans(Material):
         .. code-block:: python
 
             {
-            "type": "trans",  # Material type
-            "identifier": "", // Material identifier
-            "display_name": string  # Material display name
+            "type": "Trans",  # Material type
+            "identifier": "", # Material identifier
+            "display_name": "",  # Material display name
             "r_reflectance": float,  # Reflectance for red
             "g_reflectance": float,  # Reflectance for green
             "b_reflectance": float,  # Reflectance for blue
@@ -365,13 +459,8 @@ class Trans(Material):
             "modifier": {},  # Material modifier (Default: None)
             }
         """
-        assert 'type' in data, 'Input dictionary is missing "type".'
-        if data['type'] != cls.__name__.lower():
-            raise ValueError(
-                'Type must be %s not %s.' % (cls.__name__.lower(),
-                                             data['type'])
-            )
-        modifier, dependencies = Material.filter_dict_input(data)
+        cls._dict_type_check(cls.__name__, data)
+        modifier, dependencies = cls.filter_dict_input(data)
 
         new_obj = cls(
             identifier=data["identifier"],
@@ -392,7 +481,7 @@ class Trans(Material):
         """Translate this object to a dictionary."""
         base = {
             'modifier': self.modifier.to_dict(),
-            'type': self.__class__.__name__.lower(),
+            'type': 'Trans',
             'identifier': self.identifier,
             'r_reflectance': self.r_reflectance,
             'g_reflectance': self.g_reflectance,
