@@ -47,7 +47,7 @@ _default_modifiers = {
 
 @lockable
 class ModifierSet(object):
-    """Set containting all radiance modifiers needed to create a radiance model.
+    """Set containing all radiance modifiers needed to create a radiance model.
 
     ModifierSets can be used to establish templates that are applied broadly
     across a Model, like a color scheme used consistently throughout a building.
@@ -340,7 +340,7 @@ class ModifierSet(object):
             raise NotImplementedError(
                 'Boundary condition {} is not recognized for doors in '
                 'ModifierSet'.format(boundary_condition)
-                )
+            )
 
     def get_shade_modifier(self, is_indoor=False):
         """Get a modifier object that will be assigned to a shade.
@@ -979,15 +979,16 @@ class DoorModifierSet(_BaseSet):
     """Set containing all radiance modifiers needed to for an radiance model's Doors.
 
     Args:
-        exterior_modifier: A window modifier object for apertures
-            with an Outdoors boundary condition.
-        interior_modifier: A window modifier object for apertures
-            with a Surface boundary condition.
-        exterior_glass_modifier:
-        interior_glass_modifier:
-        overhead_modifier: : A window modifier object for doors with an
-            Outdoors boundary condition and a RoofCeiling or Floor face type for
-            their parent face.
+        exterior_modifier: A modifier object for doors in walls with an Outdoors
+            boundary condition.
+        interior_modifier: A modifier object for doors with a Surface (or other)
+            boundary condition.
+        exterior_glass_modifier: A modifier object for all glass doors with an
+            Outdoors boundary condition.
+        interior_glass_modifier: A modifier object for all glass doors with a
+            Surface boundary condition.
+        overhead_modifier: : A modifier object for doors with an Outdoors boundary
+            condition and a RoofCeiling or Floor face type for their parent face.
 
     Properties:
         * exterior_modifier
@@ -1087,5 +1088,4 @@ class DoorModifierSet(_BaseSet):
 
     def __repr__(self):
         return 'Door Modifier Set: [Exterior: {}] [Interior: {}]'.format(
-                self.exterior_modifier.display_name,
-                self.interior_modifier.display_name)
+            self.exterior_modifier.display_name, self.interior_modifier.display_name)
