@@ -1,16 +1,18 @@
 # coding=utf-8
 """Model Radiance Properties."""
+from honeybee.extensionutil import model_extension_dicts
+from honeybee.checkdup import check_duplicate_identifiers
+from honeybee.boundarycondition import Surface
+from honeybee.typing import invalid_dict_error, clean_rad_string
+
 from ..sensorgrid import SensorGrid
 from ..view import View
 from ..dynamic.group import DynamicShadeGroup, DynamicSubFaceGroup
 from ..modifierset import ModifierSet
 from ..mutil import dict_to_modifier  # imports all modifiers classes
-from ..modifier.material import BSDF
+from ..modifier.material import aBSDF, BSDF
 from ..lib.modifiers import black, generic_context
-
-from honeybee.extensionutil import model_extension_dicts
-from honeybee.checkdup import check_duplicate_identifiers
-from honeybee.boundarycondition import Surface
+from ..lib.modifiersets import generic_modifier_set_visible
 
 try:
     from itertools import izip as zip  # python 2
@@ -35,6 +37,7 @@ class ModelRadianceProperties(object):
         * shade_modifiers
         * bsdf_modifiers
         * modifier_sets
+        * global_modifier_set
         * dynamic_shade_groups
         * dynamic_subface_groups
         * shade_group_identifiers
@@ -145,9 +148,9 @@ class ModelRadianceProperties(object):
         for face in self.host.faces:  # check all orphaned Face modifiers
             self._check_and_add_face_modifier(face, modifiers)
         for ap in self.host.orphaned_apertures:  # check all Aperture modifiers
-            self._check_and_add_obj_modifier(ap, modifiers)
+            self._check_and_add_dynamic_obj_modifier(ap, modifiers)
         for dr in self.host.orphaned_doors:  # check all Door modifiers
-            self._check_and_add_obj_modifier(dr, modifiers)
+            self._check_and_add_dynamic_obj_modifier(dr, modifiers)
         return list(set(modifiers))
 
     @property
@@ -174,7 +177,7 @@ class ModelRadianceProperties(object):
         Model.blk_modifiers.
         """
         all_mods = self.modifiers + self.blk_modifiers
-        return list(set(mod for mod in all_mods if isinstance(mod, BSDF)))
+        return list(set(mod for mod in all_mods if isinstance(mod, (aBSDF, BSDF))))
 
     @property
     def modifier_sets(self):
@@ -187,6 +190,16 @@ class ModelRadianceProperties(object):
                     modifier_sets.append(room.properties.radiance._modifier_set)
         return list(set(modifier_sets))  # catch equivalent modifier sets
 
+    @property
+    def global_modifier_set(self):
+        """The global radiance modifier set.
+
+        This is what is used whenever no modifier has been assigned to a given
+        Face/Aperture/Door/Shade and there is no modifier_set assigned to the
+        parent Room.
+        """
+        return generic_modifier_set_visible
+
     @property
     def dynamic_shade_groups(self):
         """Get a list of DynamicShadeGroups in the model.
@@ -225,7 +238,7 @@ class ModelRadianceProperties(object):
 
     @property
     def shade_group_identifiers(self):
-        """Get a list of identifers for all the DynamicShadeGroups in the model."""
+        """Get a list of identifiers for all the DynamicShadeGroups in the model."""
         group_ids = set()
         for shade in self.host.shades:
             if shade.properties.radiance._dynamic_group_identifier:
@@ -234,7 +247,7 @@ class ModelRadianceProperties(object):
 
     @property
     def subface_group_identifiers(self):
-        """Get a list of identifers for all the DynamicSubFaceGroups in the model."""
+        """Get a list of identifiers for all the DynamicSubFaceGroups in the model."""
         group_ids = set()
         for subface in self.host.apertures + self.host.doors:
             if subface.properties.radiance._dynamic_group_identifier:
@@ -292,23 +305,35 @@ class ModelRadianceProperties(object):
         """Get all Faces in the model separated by their blk property.
 
         This method will also ensure that any faces with Surface boundary condition
-        only have one of such objects in the output lists.
+        are not duplicated in the result but are rather offset from the base face
+        to ensure modifiers on opposite sides of interior Faces are accounted for.
+        Furthermore, this method also ensures that any static interior sub-faces
+        (with Surface
+        boundary condition) only have one of such objects in the output lists.
 
         Returns:
             A tuple with 2 lists:
 
-            -   faces: A list of all  faces without a unique modifier_blk.
+            -   faces: A list of all faces without a unique modifier_blk.
 
             -   faces_blk: A list of all opaque faces that have a unique modifier_blk.
         """
         faces = []
         faces_blk = []
-        interior_faces = set()
+        interior_faces, offset = set(), self.host.tolerance * -2
         for face in self.host.faces:
             if isinstance(face.boundary_condition, Surface):
                 if face.identifier in interior_faces:
-                    continue
-                interior_faces.add(face.boundary_condition.boundary_condition_object)
+                    face = face.duplicate()
+                    face.move(face.normal * offset)
+                else:
+                    interior_faces.add(face.boundary_condition.boundary_condition_object)
+                    for subf in face.apertures + face.doors:
+                        if subf.properties.radiance.dynamic_group_identifier is None:
+                            if subf.properties.radiance._modifier_blk:
+                                faces_blk.append(subf)
+                            else:
+                                faces.append(subf)
             if face.properties.radiance._modifier_blk:
                 faces_blk.append(face)
             else:
@@ -316,11 +341,9 @@ class ModelRadianceProperties(object):
         return faces, faces_blk
 
     def subfaces_by_blk(self):
-        """Get model sub-faces (Apertures, Doors) separated by their blk property.
+        """Get model exterior sub-faces (Apertures, Doors) grouped by their blk property.
 
         Dynamic sub-faces will be excluded from the output lists.
-        This method will also ensure that any interior sub-faces (with Surface
-        boundary condition) only have one of such objects in the output lists.
 
         Returns:
             A tuple with 2 lists:
@@ -332,14 +355,11 @@ class ModelRadianceProperties(object):
         """
         subfaces = []
         subfaces_blk = []
-        interior_ids = set()
         for subf in self.host.apertures + self.host.doors:
             if subf.properties.radiance.dynamic_group_identifier:
                 continue  # sub-face will be accounted for in the dynamic objects
             if isinstance(subf.boundary_condition, Surface):
-                if subf.identifier in interior_ids:
-                    continue
-                interior_ids.add(subf.boundary_condition.boundary_condition_object)
+                continue  # static interior apertures are part of the scene
             if subf.properties.radiance._modifier_blk:
                 subfaces_blk.append(subf)
             else:
@@ -434,56 +454,286 @@ class ModelRadianceProperties(object):
         for view in self._views:
             view.scale(factor, origin)
 
-    def check_duplicate_modifier_identifiers(self, raise_exception=True):
-        """Check that there are no duplicate Modifier identifiers in the model."""
+    def generate_exterior_face_sensor_grid(
+            self, dimension, offset=0.1, face_type='Wall', punched_geometry=False):
+        """Get a radiance SensorGrid generated from all exterior Faces of this Model.
+
+        The Face geometry without windows punched into it will be used. This
+        will be None if the Model has no exterior Faces.
+
+        Args:
+            dimension: The dimension of the grid cells as a number.
+            offset: A number for how far to offset the grid from the base face.
+                Positive numbers indicate an offset towards the exterior. (Default
+                is 0.1, which will offset the grid to be 0.1 unit from the faces).
+            face_type: Text to specify the type of face that will be used to
+                generate grids. Note that only Faces with Outdoors boundary
+                conditions will be used, meaning that most Floors will typically
+                be excluded unless they represent the underside of a cantilever.
+                Choose from the following. (Default: Wall).
+
+                * Wall
+                * Roof
+                * Floor
+                * All
+
+            punched_geometry: Boolean to note whether the punched_geometry of the faces
+                should be used (True) with the areas of sub-faces removed from the grid
+                or the full geometry should be used (False). (Default:False).
+
+        Returns:
+            A honeybee_radiance SensorGrid generated from the exterior Faces
+            of the model. Will be None if the Model has no exterior Faces.
+        """
+        # generate the mesh grid from the exterior Faces
+        face_grid = self.host.generate_exterior_face_grid(
+            dimension, offset, face_type, punched_geometry)
+        if face_grid is None:  # no valid mesh could be generated
+            return None
+        # create the sensor grid from the mesh
+        f_nm = 'Faces' if face_type.title() == 'All' else face_type.title()
+        grid_name = '{}_Exterior{}'.format(self.host.display_name, f_nm)
+        sensor_grid = SensorGrid.from_mesh3d(clean_rad_string(grid_name), face_grid)
+        sensor_grid.display_name = grid_name
+        return sensor_grid
+
+    def generate_exterior_aperture_sensor_grid(
+            self, dimension, offset=0.1, aperture_type='All'):
+        """Get a radiance SensorGrid generated from all exterior Apertures of this Model.
+
+        This will be None if the Model has no exterior Apertures.
+
+        Args:
+            dimension: The dimension of the grid cells as a number.
+            offset: A number for how far to offset the grid from the base aperture.
+                Positive numbers indicate an offset towards the exterior while
+                negative numbers indicate an offset towards the interior, essentially
+                modeling the value of sun on the building interior. (Default
+                is 0.1, which will offset the grid to be 0.1 unit from the aperture).
+            aperture_type: Text to specify the type of Aperture that will be used to
+                generate grids. Window indicates Apertures in Walls. Choose from
+                the following. (Default: All).
+
+                * Window
+                * Skylight
+                * All
+
+        Returns:
+            A honeybee_radiance SensorGrid generated from the exterior Apertures
+            of the model. Will be None if the Model has no exterior Apertures.
+        """
+        # generate the mesh grid from the exterior Apertures
+        ap_grid = self.host.generate_exterior_aperture_grid(
+            dimension, offset, aperture_type)
+        if ap_grid is None:  # no valid mesh could be generated
+            return None
+        # create the sensor grid from the mesh
+        f_nm = 'Apertures' if aperture_type.title() == 'All' else aperture_type.title()
+        grid_name = '{}_Exterior{}'.format(self.host.display_name, f_nm)
+        sensor_grid = SensorGrid.from_mesh3d(clean_rad_string(grid_name), ap_grid)
+        sensor_grid.display_name = grid_name
+        return sensor_grid
+
+    def check_all(self, raise_exception=True, detailed=False):
+        """Check all of the aspects of the Model radiance properties.
+
+        Args:
+            raise_exception: Boolean to note whether a ValueError should be raised
+                if any errors are found. If False, this method will simply
+                return a text string with all errors that were found. (Default: True).
+            detailed: Boolean for whether the returned object is a detailed list of
+                dicts with error info or a string with a message. (Default: False).
+
+        Returns:
+            A text string with all errors that were found or a list if detailed is True.
+            This string (or list) will be empty if no errors were found.
+        """
+        # set up defaults to ensure the method runs correctly
+        detailed = False if raise_exception else detailed
+        msgs = []
+        # perform checks for key honeybee model schema rules
+        msgs.append(self.check_duplicate_modifier_identifiers(False, detailed))
+        msgs.append(self.check_duplicate_modifier_set_identifiers(False, detailed))
+        msgs.append(self.check_duplicate_sensor_grid_identifiers(False, detailed))
+        msgs.append(self.check_duplicate_view_identifiers(False, detailed))
+        msgs.append(self.check_sensor_grid_rooms_in_model(False, detailed))
+        msgs.append(self.check_view_rooms_in_model(False, detailed))
+        # output a final report of errors or raise an exception
+        full_msgs = [msg for msg in msgs if msg]
+        if detailed:
+            return [m for msg in full_msgs for m in msg]
+        full_msg = '\n'.join(full_msgs)
+        if raise_exception and len(full_msgs) != 0:
+            raise ValueError(full_msg)
+        return full_msg
+
+    def check_duplicate_modifier_identifiers(self, raise_exception=True, detailed=False):
+        """Check that there are no duplicate Modifier identifiers in the model.
+
+        Args:
+            raise_exception: Boolean to note whether a ValueError should be raised
+                if duplicate identifiers are found. (Default: True).
+            detailed: Boolean for whether the returned object is a detailed list of
+                dicts with error info or a string with a message. (Default: False).
+
+        Returns:
+            A string with the message or a list with a dictionary if detailed is True.
+        """
         return check_duplicate_identifiers(
-            self.modifiers, raise_exception, 'Radiance Modifier')
+            self.modifiers, raise_exception, 'Radiance Modifier',
+            detailed, '010001', 'Radiance', error_type='Duplicate Modifier Identifier')
 
-    def check_duplicate_modifier_set_identifiers(self, raise_exception=True):
-        """Check that there are no duplicate ModifierSet identifiers in the model."""
+    def check_duplicate_modifier_set_identifiers(
+            self, raise_exception=True, detailed=False):
+        """Check that there are no duplicate ModifierSet identifiers in the model.
+
+        Args:
+            raise_exception: Boolean to note whether a ValueError should be raised
+                if duplicate identifiers are found. (Default: True).
+            detailed: Boolean for whether the returned object is a detailed list of
+                dicts with error info or a string with a message. (Default: False).
+
+        Returns:
+            A string with the message or a list with a dictionary if detailed is True.
+        """
         return check_duplicate_identifiers(
-            self.modifier_sets, raise_exception, 'ModifierSet')
+            self.modifier_sets, raise_exception, 'ModifierSet',
+            detailed, '010002', 'Radiance',
+            error_type='Duplicate ModifierSet Identifier')
 
-    def check_duplicate_sensor_grid_identifiers(self, raise_exception=True):
-        """Check that there are no duplicate SensorGrid identifiers in the model."""
+    def check_duplicate_sensor_grid_identifiers(
+            self, raise_exception=True, detailed=False):
+        """Check that there are no duplicate SensorGrid identifiers in the model.
+
+        Args:
+            raise_exception: Boolean to note whether a ValueError should be raised
+                if duplicate identifiers are found. (Default: True).
+            detailed: Boolean for whether the returned object is a detailed list of
+                dicts with error info or a string with a message. (Default: False).
+
+        Returns:
+            A string with the message or a list with a dictionary if detailed is True.
+        """
         return check_duplicate_identifiers(
-            self.sensor_grids, raise_exception, 'SensorGrid')
+            self.sensor_grids, raise_exception, 'SensorGrid',
+            detailed, '010003', 'Radiance', error_type='Duplicate SensorGrid Identifier')
 
-    def check_duplicate_view_identifiers(self, raise_exception=True):
-        """Check that there are no duplicate View identifiers in the model."""
-        return check_duplicate_identifiers(self.views, raise_exception, 'View')
+    def check_duplicate_view_identifiers(self, raise_exception=True, detailed=False):
+        """Check that there are no duplicate View identifiers in the model.
 
-    def check_sensor_grid_rooms_in_model(self, raise_exception=True):
-        """Check that the room_identifiers of SenorGrids are in the model."""
-        grid_ids = [grid.room_identifier for grid in self.sensor_grids]
+        Args:
+            raise_exception: Boolean to note whether a ValueError should be raised
+                if duplicate identifiers are found. (Default: True).
+            detailed: Boolean for whether the returned object is a detailed list of
+                dicts with error info or a string with a message. (Default: False).
+
+        Returns:
+            A string with the message or a list with a dictionary if detailed is True.
+        """
+        return check_duplicate_identifiers(
+            self.views, raise_exception, 'View', detailed, '010004', 'Radiance',
+            error_type='Duplicate View Identifier')
+
+    def check_sensor_grid_rooms_in_model(self, raise_exception=True, detailed=False):
+        """Check that the room_identifiers of SenorGrids are in the model.
+
+        Args:
+            raise_exception: Boolean to note whether a ValueError should be raised if
+                SensorGrids reference Rooms that are not in the Model. (Default: True).
+            detailed: Boolean for whether the returned object is a detailed list of
+                dicts with error info or a string with a message. (Default: False).
+
+        Returns:
+            A string with the message or a list with a dictionary if detailed is True.
+        """
+        detailed = False if raise_exception else detailed
+        # gather a list of all the missing rooms
+        grid_ids = [(grid, grid.room_identifier) for grid in self.sensor_grids
+                    if grid.room_identifier is not None]
         room_ids = set(room.identifier for room in self.host.rooms)
-        missing_rooms = set()
+        missing_rooms = [] if detailed else set()
         for grid in grid_ids:
-            if grid not in room_ids:
-                missing_rooms.add(grid)
+            if grid[1] not in room_ids:
+                if detailed:
+                    missing_rooms.append(grid[0])
+                else:
+                    missing_rooms.add(grid[1])
+        # if missing rooms were found, then report the issue
         if len(missing_rooms) != 0:
-            if raise_exception:
-                raise ValueError(
-                    'The model has the following missing rooms referenced by sensor '
-                    'grids:\n{}'.format('\n'.join(missing_rooms)))
-            return False
-        return True
+            if detailed:
+                all_err = []
+                for grid in missing_rooms:
+                    msg = 'SensorGrid "{}" has a room_identifier that is not in the ' \
+                        'Model: "{}"'.format(grid.identifier, grid.room_identifier)
+                    error_dict = {
+                        'type': 'ValidationError',
+                        'code': '010005',
+                        'error_type': 'SensorGrid Room Not In Model',
+                        'extension_type': 'Radiance',
+                        'element_type': 'SensorGrid',
+                        'element_id': grid.identifier,
+                        'element_name': grid.display_name,
+                        'message': msg
+                    }
+                    all_err.append(error_dict)
+                return all_err
+            else:
+                msg = 'The model has the following missing rooms referenced by sensor ' \
+                    'grids:\n{}'.format('\n'.join(missing_rooms))
+                if raise_exception:
+                    raise ValueError(msg)
+                return msg
+        return [] if detailed else ''
 
-    def check_view_rooms_in_model(self, raise_exception=True):
-        """Check that the room_identifiers of Views are in the model."""
-        view_ids = [view.room_identifier for view in self.views]
+    def check_view_rooms_in_model(self, raise_exception=True, detailed=False):
+        """Check that the room_identifiers of Views are in the model.
+
+        Args:
+            raise_exception: Boolean to note whether a ValueError should be raised if
+                Views reference Rooms that are not in the Model. (Default: True).
+            detailed: Boolean for whether the returned object is a detailed list of
+                dicts with error info or a string with a message. (Default: False).
+
+        Returns:
+            A string with the message or a list with a dictionary if detailed is True.
+        """
+        detailed = False if raise_exception else detailed
+        # gather a list of all the missing rooms
+        view_ids = [(view, view.room_identifier) for view in self.views
+                    if view.room_identifier is not None]
         room_ids = set(room.identifier for room in self.host.rooms)
-        missing_rooms = set()
+        missing_rooms = [] if detailed else set()
         for view in view_ids:
-            if view not in room_ids:
-                missing_rooms.add(view)
+            if view[1] not in room_ids:
+                if detailed:
+                    missing_rooms.append(view[0])
+                else:
+                    missing_rooms.add(view[1])
         if len(missing_rooms) != 0:
-            if raise_exception:
-                raise ValueError(
-                    'The model has the following missing rooms referenced by '
-                    'views:\n{}'.format('\n'.join(missing_rooms)))
-            return False
-        return True
+            if detailed:
+                all_err = []
+                for view in missing_rooms:
+                    msg = 'View "{}" has a room_identifier that is not in the ' \
+                        'Model: "{}"'.format(view.identifier, view.room_identifier)
+                    error_dict = {
+                        'type': 'ValidationError',
+                        'code': '010006',
+                        'error_type': 'View Room Not In Model',
+                        'extension_type': 'Radiance',
+                        'element_type': 'View',
+                        'element_id': view.identifier,
+                        'element_name': view.display_name,
+                        'message': msg
+                    }
+                    all_err.append(error_dict)
+                return all_err
+            else:
+                msg = 'The model has the following missing rooms referenced by ' \
+                    'views:\n{}'.format('\n'.join(missing_rooms))
+                if raise_exception:
+                    raise ValueError(msg)
+                return msg
+        return [] if detailed else ''
 
     def apply_properties_from_dict(self, data):
         """Apply the radiance properties of a dictionary to the host Model of this object.
@@ -536,6 +786,16 @@ class ModelRadianceProperties(object):
         """Return Model radiance properties as a dictionary."""
         base = {'radiance': {'type': 'ModelRadianceProperties'}}
 
+        # add the global modifier set to the dictionary
+        gs = self.global_modifier_set.to_dict(abridged=True, none_for_defaults=False)
+        gs['type'] = 'GlobalModifierSet'
+        del gs['identifier']
+        g_mods = self.global_modifier_set.modifiers_unique
+        gs['modifiers'] = [mod.to_dict() for mod in g_mods]
+        gs['context_modifier'] = generic_context.identifier
+        gs['modifiers'].append(generic_context.to_dict())
+        base['radiance']['global_modifier_set'] = gs
+
         # add all ModifierSets to the dictionary
         base['radiance']['modifier_sets'] = []
         modifier_sets = self.modifier_sets
@@ -568,7 +828,9 @@ class ModelRadianceProperties(object):
             If None, the properties will be duplicated with the same host.
         """
         _host = new_host or self._host
-        return ModelRadianceProperties(_host, self._sensor_grids, self._views)
+        new_grids = [sg.duplicate() for sg in self._sensor_grids]
+        new_views = [vw.duplicate() for vw in self._views]
+        return ModelRadianceProperties(_host, new_grids, new_views)
 
     @staticmethod
     def load_properties_from_dict(data):
@@ -599,22 +861,60 @@ class ModelRadianceProperties(object):
 
         # process all modifiers in the ModelRadianceProperties dictionary
         modifiers = {}
-        for mod in data['properties']['radiance']['modifiers']:
-            modifiers[mod['identifier']] = dict_to_modifier(mod)
+        if 'modifiers' in data['properties']['radiance'] and \
+                data['properties']['radiance']['modifiers'] is not None:
+            for mod in data['properties']['radiance']['modifiers']:
+                try:
+                    modifiers[mod['identifier']] = dict_to_modifier(mod)
+                except Exception as e:
+                    invalid_dict_error(mod, e)
 
         # process all modifier sets in the ModelRadianceProperties dictionary
         modifier_sets = {}
         if 'modifier_sets' in data['properties']['radiance'] and \
                 data['properties']['radiance']['modifier_sets'] is not None:
-            for m_set in data['properties']['radiance']['modifier_sets']:
-                if m_set['type'] == 'ModifierSet':
-                    modifier_sets[m_set['identifier']] = ModifierSet.from_dict(m_set)
-                else:
-                    modifier_sets[m_set['identifier']] = \
-                        ModifierSet.from_dict_abridged(m_set, modifiers)
+            if 'modifier_sets' in data['properties']['radiance'] and \
+                    data['properties']['radiance']['modifier_sets'] is not None:
+                for m_set in data['properties']['radiance']['modifier_sets']:
+                    try:
+                        if m_set['type'] == 'ModifierSet':
+                            modifier_sets[m_set['identifier']] = \
+                                ModifierSet.from_dict(m_set)
+                        else:
+                            modifier_sets[m_set['identifier']] = \
+                                ModifierSet.from_dict_abridged(m_set, modifiers)
+                    except Exception as e:
+                        invalid_dict_error(m_set, e)
 
         return modifiers, modifier_sets
 
+    @staticmethod
+    def dump_properties_to_dict(modifiers=None, modifier_sets=None):
+        """Get a ModelRadianceProperties dictionary from arrays of Python objects.
+
+        Args:
+            modifiers: A list or tuple of radiance modifier objects.
+            modifier_sets: A list or tuple of modifier set objects.
+
+        Returns:
+            data: A dictionary representation of ModelRadianceProperties. Note that
+                all objects in this dictionary will follow the abridged schema.
+        """
+        # process the modifiers and modifier sets
+        all_m = [] if modifiers is None else list(modifiers)
+        all_mod_sets = [] if modifier_sets is None else list(modifier_sets)
+        for mod_set in all_mod_sets:
+            all_m.extend(mod_set.modified_modifiers)
+
+        # get sets of unique objects
+        all_mods = set(all_m)
+
+        # add all object dictionaries into one object
+        data = {'type': 'ModelRadianceProperties'}
+        data['modifiers'] = [m.to_dict() for m in all_mods]
+        data['modifier_sets'] = [ms.to_dict(abridged=True) for ms in all_mod_sets]
+        return data
+
     def _check_and_add_room_modifier_shade(self, room, modifiers):
         """Check if a modifier is assigned to a Room's shades and add it to a list."""
         self._check_and_add_obj_modifier_shade(room, modifiers)
