@@ -4,12 +4,15 @@ from __future__ import division
 from .sensor import Sensor
 from .lightpath import light_path_from_room
 
+from honeybee.facetype import AirBoundary
 import honeybee.typing as typing
 import ladybug.futil as futil
+from ladybug_geometry.geometry3d.pointvector import Point3D, Vector3D
 from ladybug_geometry.geometry3d.face import Face3D
 from ladybug_geometry.geometry3d.mesh import Mesh3D
 
 import os
+import json
 import math
 try:
     from itertools import izip as zip
@@ -37,6 +40,7 @@ class SensorGrid(object):
         * mesh
         * base_geometry
         * group_identifier
+        * full_identifier
 
     """
 
@@ -97,10 +101,10 @@ class SensorGrid(object):
             identifier: Text string for a unique SensorGrid ID. Must not contain spaces
                 or special characters. This will be used to identify the object across
                 a model and in the exported Radiance files.
-            positions: A list of (x, y ,z) for position of sensors.
-            plane_normal: (x, y, z) for direction of sensors.
+            positions: A list of (x, y ,z) tuples for position of sensors.
+            plane_normal: (x, y, z) tuples for direction of sensors.
         """
-        sg = (Sensor(l, plane_normal) for l in positions)
+        sg = (Sensor(pt, plane_normal) for pt in positions)
         return cls(identifier, sg)
 
     @classmethod
@@ -114,10 +118,10 @@ class SensorGrid(object):
             identifier: Text string for a unique SensorGrid ID. Must not contain spaces
                 or special characters. This will be used to identify the object across
                 a model and in the exported Radiance files.
-            positions: A list of (x, y ,z) for position of sensors.
-            directions: A list of (x, y, z) for direction of sensors.
+            positions: A list of (x, y ,z) tuples for position of sensors.
+            directions: A list of (x, y, z) tuples for direction of sensors.
         """
-        sg = tuple(Sensor(l, v) for l, v in zip(positions, directions))
+        sg = tuple(Sensor(pt, v) for pt, v in zip(positions, directions))
         return cls(identifier, sg)
 
     @classmethod
@@ -156,7 +160,8 @@ class SensorGrid(object):
             identifier: Text string for a unique SensorGrid ID. Must not contain spaces
                 or special characters. This will be used to identify the object across
                 a model and in the exported Radiance files.
-            faces: An array of ladybug_geometry Face3D.
+            faces: An array of ladybug_geometry Face3Ds from which a SensorGrid will
+                be generated.
             x_dim: The x dimension of the grid cells as a number.
             y_dim: The y dimension of the grid cells as a number. Default is None,
                 which will assume the same cell dimension for y as is set for x.
@@ -167,7 +172,14 @@ class SensorGrid(object):
                 normal direction of the face will be used as the direction of the
                 sensor grids.
         """
-        meshes = [face.mesh_grid(x_dim, y_dim, offset, flip) for face in faces]
+        meshes = []
+        for face in faces:
+            try:
+                meshes.append(face.mesh_grid(x_dim, y_dim, offset, flip))
+            except AssertionError:  # tiny geometry not compatible with quad faces
+                continue
+        assert len(meshes) > 0, 'None of the Face3Ds input to SensorGrid.from_face3d ' \
+            'can produce a quad grid at the specified grid dimensions.'
         if len(meshes) == 1:
             s_grid = cls.from_mesh3d(identifier, meshes[0])
         elif len(meshes) > 1:
@@ -175,6 +187,81 @@ class SensorGrid(object):
         s_grid.base_geometry = faces
         return s_grid
 
+    @classmethod
+    def from_positions_radial(
+            cls, identifier, positions, dir_count=8, start_vector=Vector3D(0, -1, 0),
+            mesh_radius=0):
+        """Create a sensor grid from radial directions around sensor positions.
+
+        This type of sensor grid is particularly helpful for studies of multiple view
+        directions, such as imageless glare studies.
+
+        Args:
+            identifier: Text string for a unique SensorGrid ID. Must not contain spaces
+                or special characters. This will be used to identify the object across
+                a model and in the exported Radiance files.
+            positions: A list of (x, y ,z) tuples for position of sensors.
+            dir_count: A positive integer for the number of radial directions
+                to be generated around each position. (Default: 8).
+            start_vector: A Vector3D to set the start direction of the generated
+                directions. This can be used to orient the resulting sensors to
+                specific parts of the scene. It can also change the elevation of the
+                resulting directions since this start vector will always be rotated in
+                the XY plane to generate the resulting directions. (Default: (0, -1, 0)).
+            mesh_radius: An optional number that can be used to generate a Mesh3D
+                that is aligned with the resulting sensors and will automatically
+                be assigned to the grid's mesh property. Such meshes will resemble
+                a circle around each sensor with the specified radius and will
+                contain triangular faces that can be colored with simulation results.
+                If zero, no mesh will be generated for the sensor grid. (Default: 0).
+        """
+        # set up the vectors to generate the rays
+        inc_ang = (math.pi * 2) / dir_count
+        vw_vecs = [start_vector.rotate_xy(i * inc_ang) for i in range(dir_count)]
+        vw_vecs = [(round(v.x, 5), round(v.y, 5), round(v.z, 3)) for v in vw_vecs]
+        # set up the sensor grid object
+        sensors = tuple(Sensor(pt, v) for pt in positions for v in vw_vecs)
+        sg = cls(identifier, sensors)
+        # generate the mesh if it was requested
+        if mesh_radius > 0:
+            sg.mesh = cls.radial_positions_mesh(
+                positions, dir_count, start_vector, mesh_radius)
+        return sg
+
+    @classmethod
+    def from_mesh3d_radial(
+            cls, identifier, mesh, dir_count=8, start_vector=Vector3D(0, -1, 0),
+            mesh_radius=0):
+        """Create a sensor grid from radial directions around centroids of a Mesh3D.
+
+        This type of sensor grid is particularly helpful for studies of multiple view
+        directions, such as imageless glare studies.
+
+        Args:
+            identifier: Text string for a unique SensorGrid ID. Must not contain spaces
+                or special characters. This will be used to identify the object across
+                a model and in the exported Radiance files.
+            mesh: A ladybug_geometry Mesh3D from which the sensor grid will be generated.
+            dir_count: A positive integer for the number of radial directions
+                to be generated around each position. (Default: 8).
+            start_vector: A Vector3D to set the start direction of the generated
+                directions. This can be used to orient the resulting sensors to
+                specific parts of the scene. It can also change the elevation of
+                the resulting directions since this start vector will always be
+                rotated in the XY plane to generate the resulting directions.
+            mesh_radius: An optional number that can be used to generate a Mesh3D
+                that is aligned with the resulting sensors and will automatically
+                be assigned to the grid's mesh property. Such meshes will resemble
+                a circle around each sensor with the specified radius and will
+                contain triangular faces that can be colored with simulation results.
+                If zero, no mesh will be generated for the sensor grid. (Default: 0).
+        """
+        assert isinstance(mesh, Mesh3D), 'Expected ladybug_geometry Mesh3D for ' \
+            'SensorGrid.from_mesh3d. Got {}.'.format(type(mesh))
+        positions = [(pt.x, pt.y, pt.z) for pt in mesh.face_centroids]
+        return cls.from_positions_radial(
+            identifier, positions, dir_count, start_vector, mesh_radius)
+
     @classmethod
     def from_file(cls, file_path, start_line=None, end_line=None, identifier=None):
         """Create a sensor grid from a sensor (.pts) file.
@@ -268,12 +355,12 @@ class SensorGrid(object):
     @property
     def positions(self):
         """Get a generator of sensor positions as x, y, z."""
-        return (ap.position for ap in self.sensors)
+        return (ap.pos for ap in self.sensors)
 
     @property
     def directions(self):
         """Get a generator of sensor directions as x, y , z."""
-        return (ap.direction for ap in self.sensors)
+        return (ap.dir for ap in self.sensors)
 
     @property
     def count(self):
@@ -309,20 +396,30 @@ class SensorGrid(object):
         return self._group_identifier
 
     @group_identifier.setter
-    def group_identifier(self, n):
-        identifier_key = \
-            '/'.join(
-                typing.valid_rad_string(key, 'Sensor group identifier')
-                for key in n.split('/')
-            )
+    def group_identifier(self, identifier_key):
+        if identifier_key is not None:
+            identifier_key = \
+                '/'.join(
+                    typing.valid_rad_string(key, 'sensor grid group identifier')
+                    for key in identifier_key.split('/')
+                )
         self._group_identifier = identifier_key
 
+    @property
+    def full_identifier(self):
+        """Get full identifier for a sensor grid.
+
+        For a sensor grid with group identifier it will be group_identifier/identifier
+        """
+        return self.identifier if not self.group_identifier \
+            else '%s/%s' % (self.group_identifier, self.identifier)
+
     @property
     def light_path(self):
         """Get or set list of lists for the light path from the grid to the sky.
 
         Each sub-list contains identifiers of aperture groups through which light
-        passes. (eg. [['SouthWindow1'], ['static_apertures', 'NorthWindow2']]).
+        passes. (eg. [['SouthWindow1'], ['__static_apertures__', 'NorthWindow2']]).
         Setting this property will override any auto-calculation of the light
         path from the model upon export to the simulation.
         """
@@ -391,17 +488,109 @@ class SensorGrid(object):
             model: A honeybee Model object which will be used to identify
                 the aperture groups that will be run with this sensor grid.
         """
-        base = {'count': self.count}
+        base = {
+            'count': self.count,
+            'name': self.display_name,
+            'identifier': self.identifier,
+            'group': self.group_identifier or '',
+            'full_id': self.full_identifier
+        }
         if self._light_path:
             base['light_path'] = self._light_path
         elif model and self._room_identifier:  # auto-calculate the light path
-            base['light_path'] = light_path_from_room(model, self._room_identifier)
+            try:
+                base['light_path'] = light_path_from_room(model, self._room_identifier)
+            except ValueError:  # room is not in the model; just ignore light path
+                pass
 
         if self._group_identifier:
             base['group_identifier'] = self._group_identifier
 
         return base
 
+    def enclosure_info_dict(self, model, air_boundary_distance=0):
+        """Get a dictionary with information about sensor relation to rooms.
+
+        This can be written as a JSON in order to map sensors with appropriate
+        energy simulation results in thermal mapping workflows.
+
+        Args:
+            model: A honeybee Model object which will be used to identify
+                the rooms/enclosure that each sensor in the grid is contained within.
+            air_boundary_distance: An optional number to set the distance from
+                air boundaries over which values should be interpolated.
+                Using 0 will assume a hard edge between Rooms of the same
+                radiant enclosures. (Default: 0).
+        """
+        # setup rooms and lists to check enclosure info
+        enclosures, sensor_indices, air_bound_proximity = {}, [], {}
+        has_indoor, has_outdoor = False, False
+        rooms = model._rooms
+        if self.room_identifier:  # put the assigned room first for faster calculation
+            rooms = model.rooms_by_identifier([self.room_identifier]) + rooms
+
+        # have a dictionary to track proximity to AirBoundary faces
+        model_ab = {}
+        for room in rooms:
+            model_ab[room.identifier] = \
+                [f for f in room.faces if isinstance(f.type, AirBoundary)]
+
+        def _air_boundary_info(distance, face, room_index):
+            """Method to perform interpolation across AirBoundary Faces."""
+            adj_room = face.boundary_condition.boundary_condition_objects[-1]
+            try:
+                adj_i = enclosures[adj_room]
+            except KeyError:  # the first time that this room is needed
+                adj_i = len(enclosures)
+                enclosures[adj_room] = len(enclosures)
+            fac_1 = 0.5 + (distance / (air_boundary_distance * 2))
+            return {room_index: fac_1, adj_i: 1 - fac_1}
+
+        # loop through the sensors and verify the room that they belong to
+        for i, sensor in enumerate(self.sensors):
+            sensor_pt = Point3D(*sensor.pos)
+            for room in rooms:
+                if room.geometry.is_point_inside(sensor_pt):
+                    # add the room index of the sensor
+                    try:
+                        sensor_indices.append(enclosures[room.identifier])
+                    except KeyError:  # the first time that this room is needed
+                        enclosures[room.identifier] = len(enclosures)
+                        sensor_indices.append(enclosures[room.identifier])
+                    has_indoor = True
+                    # test if the sensor is near any AriBoundary faces
+                    air_b = model_ab[room.identifier]
+                    if air_boundary_distance > 0 and len(air_b) != 0:
+                        for face in air_b:
+                            fg = face.geometry
+                            close_pt = fg._plane.closest_point(sensor_pt)
+                            p_dist = sensor_pt.distance_to_point(close_pt)
+                            if p_dist <= air_boundary_distance:
+                                close_pt_2d = fg._plane.xyz_to_xy(close_pt)
+                                g_dist = fg.polygon2d.distance_to_point(close_pt_2d)
+                                f_dist = math.sqrt(p_dist ** 2 + g_dist ** 2)
+                                if f_dist <= air_boundary_distance:
+                                    ab_info = _air_boundary_info(
+                                        f_dist, face, sensor_indices[-1])
+                                    try:
+                                        air_bound_proximity[i].append(ab_info)
+                                    except KeyError:
+                                        air_bound_proximity[i] = [ab_info]
+                    break  # we found the room and we don't need to iterate
+            else:  # the sensor is completely outside and not a part of a room
+                sensor_indices.append(-1)
+                has_outdoor = True
+
+        # write out the enclosure info JSON
+        mapper = sorted(enclosures, key=enclosures.__getitem__)
+        return {
+            'has_indoor': has_indoor,
+            'has_outdoor': has_outdoor,
+            'mapper': mapper,
+            'sensor_indices': sensor_indices,
+            'air_bound_proximity': air_bound_proximity
+        }
+
     def to_radiance(self):
         """Return sensors grid as a Radiance string."""
         return "\n".join((ap.to_radiance() for ap in self._sensors))
@@ -512,8 +701,69 @@ class SensorGrid(object):
             base['mesh'] = self._mesh.to_dict()
         if self._base_geometry is not None:
             base['base_geometry'] = [face.to_dict() for face in self._base_geometry]
+        if self._group_identifier is not None:
+            base['group_identifier'] = self.group_identifier
         return base
 
+    def to_json(self, folder, file_name=None, mkdir=False, ignore_group=False):
+        """Write this sensor grid to a JSON file.
+
+        Args:
+            folder: Target folder. If grid is part of a sensor group identifier it will
+                be written to a subfolder with group identifier name.
+            file_name: Optional file name without extension. (Default: self.identifier)
+            mkdir: A boolean to indicate if the folder should be created in case it
+                doesn't exist already. (Default: False).
+            ignore_group: A boolean to indicate if creating a new subfolder for sensor
+                group should be ignored. (Default: False).
+
+        Returns:
+            Full path to newly created file.
+        """
+        identifier = file_name or self.identifier + '.json'
+        if not identifier.endswith('.json'):
+            identifier += '.json'
+        if not ignore_group and self.group_identifier:
+            folder = os.path.normpath(os.path.join(folder, self.group_identifier))
+            mkdir = True  # in most cases the subfolder does not exist already
+        return futil.write_to_file_by_name(
+            folder, identifier, json.dumps(self.to_dict()), mkdir)
+
+    def to_radial_grid(self, dir_count=8, start_vector=Vector3D(0, -1, 0),
+                       mesh_radius=0):
+        """Get a radial sensor grid using the positions of this grid as a base.
+
+        All properties of this grid will be transferred to the new grid, including
+        the identifier, room_identifier, etc. The mesh will be recomputed based
+        on the input mesh_radius but any base_geometry will be transferred.
+
+        Note that calling this method on a SensorGrid that is already formatted
+        as a radial grid will result in a lot of unwanted duplication of sensors.
+
+        Args:
+            dir_count: A positive integer for the number of radial directions
+                to be generated around each position. (Default: 8).
+            start_vector: A Vector3D to set the start direction of the generated
+                directions. This can be used to orient the resulting sensors to
+                specific parts of the scene. It can also change the elevation of the
+                resulting directions since this start vector will always be rotated in
+                the XY plane to generate the resulting directions. (Default: (0, -1, 0)).
+            mesh_radius: An optional number that can be used to generate a Mesh3D
+                that is aligned with the resulting sensors and will automatically
+                be assigned to the grid's mesh property. Such meshes will resemble
+                a circle around each sensor with the specified radius and will
+                contain triangular faces that can be colored with simulation results.
+                If zero, no mesh will be generated for the sensor grid. (Default: 0).
+        """
+        new_grid = SensorGrid.from_positions_radial(
+            self.identifier, self.positions, dir_count, start_vector, mesh_radius)
+        new_grid._display_name = self._display_name
+        new_grid._room_identifier = self._room_identifier
+        new_grid.group_identifier = self.group_identifier
+        new_grid._light_path = self._light_path
+        new_grid._base_geometry = self._base_geometry
+        return new_grid
+
     def move(self, moving_vec):
         """Move this sensor grid along a vector.
 
@@ -599,6 +849,78 @@ class SensorGrid(object):
         """Get a copy of this object."""
         return self.__copy__()
 
+    @staticmethod
+    def from_face3d_arrays(
+            base_identifier, face_arrays, x_dim, y_dim=None, offset=0, flip=False):
+        """Get an array of SensorGrids from an matrix (list of lists) of Face3Ds.
+
+        This method uses the from_face3d classmethod but includes checks to
+        catch cases where not of the input Face3Ds can support the generation of
+        quad grids. In this case, the invalid SensorGrid will not be generated
+        and will be excluded form the output list of SensorGrids.
+
+        Args:
+            base_identifier: Text string for a unique SensorGrid ID, which will be used
+                as a base for all of the output SensorGrid IDs. Must not contain spaces
+                or special characters.
+            faces: An matrix (list of lists) of ladybug_geometry Face3Ds from which
+                SensorGrids will be generated.
+            x_dim: The x dimension of the grid cells as a number.
+            y_dim: The y dimension of the grid cells as a number. Default is None,
+                which will assume the same cell dimension for y as is set for x.
+            offset: A number for how far to offset the grid from the base face.
+            flip: Set to True to have the mesh normals reversed from the direction of
+                this face and to have the offset input move the mesh in the opposite
+                direction from this face normal. Defaults to False, which means the
+                normal direction of the face will be used as the direction of the
+                sensor grids.
+        """
+        grids = []
+        for i, faces in enumerate(face_arrays):
+            grid_id = '{}_{}'.format(base_identifier, i)
+            try:
+                grids.append(
+                    SensorGrid.from_face3d(grid_id, faces, x_dim, y_dim, offset, flip)
+                )
+            except AssertionError:  # none of the Face3Ds make a valid grid
+                continue
+        return grids
+
+    @staticmethod
+    def radial_positions_mesh(
+            positions, dir_count=8, start_vector=Vector3D(0, -1, 0), mesh_radius=1):
+        """Generate a Mesh3D resembling a circle around each position.
+
+        Args:
+            positions: A list of (x, y ,z) tuples for position of sensors.
+            dir_count: A positive integer for the number of radial directions
+                to be generated around each position. (Default: 8).
+            start_vector: A Vector3D to set the start direction of the generated
+                directions. (Default: (0, -1, 0)).
+            mesh_radius: A number for the radius of the radial mesh to be
+                generated around each sensor. (Default: 1).
+        """
+        # set up the start vector and rotation angles
+        st_vec = Vector3D(start_vector.x, start_vector.y, 0).normalize()
+        st_vec = st_vec * mesh_radius
+        inc_ang = (math.pi * 2) / dir_count
+        st_vec = st_vec.rotate_xy(-inc_ang / 2)
+        # loop through the positions and angles to create the mesh
+        verts, faces = [], []
+        v_count = 0
+        for pt in positions:
+            st_pt = Point3D(*pt)
+            nxt_pt = st_pt.move(st_vec)
+            verts.extend([st_pt, nxt_pt])
+            for i in range(dir_count - 1):
+                new_pt = verts[-1].rotate_xy(inc_ang, st_pt)
+                new_f = (v_count, v_count + i + 1, v_count + i + 2)
+                verts.append(new_pt)
+                faces.append(new_f)
+            faces.append((v_count, v_count + dir_count, v_count + 1))
+            v_count += (dir_count + 1)
+        return Mesh3D(verts, faces)
+
     def __len__(self):
         """Number of sensors in this grid."""
         return len(self.sensors)
