@@ -4,7 +4,6 @@ from __future__ import division
 
 from .lightpath import light_path_from_room
 
-from honeybee_radiance_command.cutil import parse_radiance_options
 from honeybee_radiance_command.options import TupleOption, \
     StringOptionJoined, NumericOption
 import honeybee.typing as typing
@@ -14,6 +13,8 @@ import ladybug.futil as futil
 
 import math
 import os
+import re
+import collections
 
 
 class View(object):
@@ -73,6 +74,8 @@ class View(object):
         * lift
         * room_identifier
         * light_path
+        * group_identifier
+        * full_identifier
 
     Usage:
 
@@ -106,7 +109,8 @@ class View(object):
 
     __slots__ = ('_identifier', '_display_name', '_position', '_direction',
                  '_up_vector', '_h_size', '_v_size', '_shift', '_lift',
-                 '_type', '_fore_clip', '_aft_clip', '_room_identifier', '_light_path')
+                 '_type', '_fore_clip', '_aft_clip', '_room_identifier',
+                 '_light_path', '_group_identifier')
 
     def __init__(self, identifier, position=None, direction=None, up_vector=None,
                  type='v', h_size=60, v_size=60, shift=None, lift=None):
@@ -134,7 +138,9 @@ class View(object):
         self._aft_clip = NumericOption('va', 'view aft clip')
 
         self._room_identifier = None
+        self._group_identifier = None
         self._light_path = None
+        self._check_size_and_type()
 
     @property
     def identifier(self):
@@ -178,7 +184,7 @@ class View(object):
             * a - Angular fisheye (a)
             * s - Planisphere [stereographic] projection (s)
         """
-        return self._type
+        return self._type.value
 
     @property
     def vt(self):
@@ -188,23 +194,7 @@ class View(object):
     @type.setter
     def type(self, value):
         self._type.value = value[-1:]  # this will handle both vtv and v inputs
-
-        # set view size to 180 degrees for fisheye views
-        if self.type in ('h', 'a', 's'):
-            if self.h_size != 180:
-                self.h_size = 180
-                print("Changed h_size to 180 for fisheye view type.")
-            if self.v_size != 180:
-                self.v_size = 180
-                print("Changed v_size to 180 for fisheye view type.")
-
-        elif self.type == 'v':
-            assert self.h_size < 180, ValueError(
-                '\n{} is an invalid horizontal view size for Perspective view.\n'
-                'The size should be smaller than 180.'.format(self.h_size))
-            assert self.v_size < 180, ValueError(
-                '\n{} is an invalid vertical view size for Perspective view.\n'
-                'The size should be smaller than 180.'.format(self.v_size))
+        self._check_size_and_type()
 
     @property
     def position(self):
@@ -213,7 +203,7 @@ class View(object):
         This is the focal point of a perspective view or the center of a parallel
         projection. Default: (0, 0, 0)
         """
-        return self._position
+        return self._position.value
 
     @property
     def vp(self):
@@ -231,7 +221,7 @@ class View(object):
         The length of this vector indicates the focal distance as needed by
         the pixel depth of field (-pd) in rpict. Default: (0, 0, 1)
         """
-        return self._direction
+        return self._direction.value
 
     @property
     def vd(self):
@@ -248,7 +238,7 @@ class View(object):
 
         Default: (0, 1, 0).
         """
-        return self._up_vector
+        return self._up_vector.value
 
     @property
     def vu(self):
@@ -263,11 +253,11 @@ class View(object):
     def h_size(self):
         """Set the view horizontal size (-vh).
 
-        For a perspective projection (including fisheye views), val is the horizontal
-        field of view (in degrees). For a parallel projection, val is the view
+        For a perspective projection (including fisheye views), this is the horizontal
+        field of view (in degrees). For a parallel projection, this is the view
         width in world coordinates.
         """
-        return self._h_size
+        return self._h_size.value
 
     @property
     def vh(self):
@@ -277,25 +267,27 @@ class View(object):
     @h_size.setter
     def h_size(self, value):
         self._h_size.value = value if value is not None else 60
+        self._check_size_and_type()
 
     @property
     def v_size(self):
         """Set the view vertical size (-vv).
 
-        For a perspective projection (including fisheye views), val is the horizontal
-        field of view (in degrees). For a parallel projection, val is the view width in
+        For a perspective projection (including fisheye views), this is the horizontal
+        field of view (in degrees). For a parallel projection, this is the view width in
         world coordinates.
         """
-        return self._v_size
+        return self._v_size.value
 
     @property
     def vv(self):
         """View vertical size as a string in radiance format."""
-        return self.v_size.to_radiance()
+        return self._v_size.to_radiance()
 
     @v_size.setter
     def v_size(self, value):
         self._v_size.value = value if value is not None else 60
+        self._check_size_and_type()
 
     @property
     def shift(self):
@@ -307,7 +299,7 @@ class View(object):
         to the right of the normal view. A value of -1 would be to the left. Larger or
         fractional values are permitted as well.
         """
-        return self._shift
+        return self._shift.value
 
     @property
     def vs(self):
@@ -324,12 +316,12 @@ class View(object):
 
         This is the amount the actual image will be lifted up from the specified view.
         """
-        return self._lift
+        return self._lift.value
 
     @property
     def vl(self):
         """View lift as a string in radiance format."""
-        return self.lift.to_radiance()
+        return self._lift.to_radiance()
 
     @lift.setter
     def lift(self, value):
@@ -348,7 +340,7 @@ class View(object):
         no foreground clipping. A negative value produces some interesting effects,
         since it creates an inverted image for objects behind the viewpoint.
         """
-        return self._fore_clip
+        return self._fore_clip.value
 
     @property
     def vo(self):
@@ -371,7 +363,7 @@ class View(object):
         no aft clipping, and is the only way to see infinitely distant objects such as
         the sky.
         """
-        return self._aft_clip
+        return self._aft_clip.value
 
     @property
     def va(self):
@@ -396,12 +388,45 @@ class View(object):
     def room_identifier(self, n):
         self._room_identifier = typing.valid_string(n)
 
+    @property
+    def group_identifier(self):
+        """Get or set text for the grid group identifier to which this SensorGrid belongs.
+
+        This will be used in the write to radiance folder method to write all the grids
+        with the same group identifier under the same subfolder.
+
+        You may use / in name to identify nested view groups. For example
+        floor_1/living_room create a view under living_room/floor_1 subfolder.
+
+        If None, the view will be written to the root of grids folder.
+        """
+        return self._group_identifier
+
+    @group_identifier.setter
+    def group_identifier(self, identifier_key):
+        if identifier_key is not None:
+            identifier_key = \
+                '/'.join(
+                    typing.valid_rad_string(key, 'view group identifier')
+                    for key in identifier_key.split('/')
+                )
+        self._group_identifier = identifier_key
+
+    @property
+    def full_identifier(self):
+        """Get full identifier for view.
+
+        For a view with group identifier it will be group_identifier/identifier
+        """
+        return self.identifier if not self.group_identifier \
+            else '%s/%s' % (self.group_identifier, self.identifier)
+
     @property
     def light_path(self):
         """Get or set list of lists for the light path from the view to the sky.
 
         Each sub-list contains identifiers of aperture groups through which light
-        passes. (eg. [['SouthWindow1'], ['static_apertures', 'NorthWindow2']]).
+        passes. (eg. [['SouthWindow1'], ['__static_apertures__', 'NorthWindow2']]).
         Setting this property will override any auto-calculation of the light
         path from the model upon export to the simulation.
         """
@@ -420,6 +445,36 @@ class View(object):
                         'aperture group identifier. Got {}.'.format(type(ap))
         self._light_path = l_path
 
+    def standardize_fisheye(self):
+        """Automatically set view size to 180 degrees if the view type is a fisheye.
+
+        Alternatively it sets the view size to 360 degrees if both the view type is
+        angular fisheye and either the horizontal or vertical view size is 360 degrees.
+        """
+        if self.type in ('h', 's'):
+            if self.h_size != 180:
+                self.h_size = 180
+            if self.v_size != 180:
+                self.v_size = 180
+        if self.type in ('a'):
+            if self.h_size == 360 or self.v_size == 360:
+                self.h_size = self.v_size = 360
+            else:
+                if self.h_size != 180:
+                    self.h_size = 180
+                if self.v_size != 180:
+                    self.v_size = 180
+
+    def _check_size_and_type(self):
+        """Check to be sure the view size and type are compatible."""
+        if self.type == 'v':
+            assert self.h_size < 180, \
+                '\n{} is an invalid horizontal view size for Perspective view.\n' \
+                'The size should be smaller than 180.'.format(self.h_size)
+            assert self.v_size < 180, \
+                '\n{} is an invalid vertical view size for Perspective view.\n' \
+                'The size should be smaller than 180.'.format(self.v_size)
+
     @classmethod
     def from_dict(cls, view_dict):
         """Create a view from a dictionary in the following format.
@@ -447,19 +502,19 @@ class View(object):
         assert view_dict['type'] == 'View', \
             'Expected View dictionary. Got {}.'.format(view_dict['type'])
 
+        view_type = view_dict['view_type'][-1:] if 'view_type' in view_dict else 'v'
         view = cls(
             identifier=view_dict['identifier'],
             position=view_dict['position'],
             direction=view_dict['direction'],
             up_vector=view_dict['up_vector'],
+            type=view_type,
             h_size=view_dict['h_size'],
             v_size=view_dict['v_size'],
             shift=view_dict['shift'],
             lift=view_dict['lift'],
         )
 
-        if 'view_type' in view_dict:
-            view.type = view_dict['view_type']
         if 'fore_clip' in view_dict:
             view.fore_clip = view_dict['fore_clip']
         if 'aft_clip' in view_dict:
@@ -470,6 +525,8 @@ class View(object):
             view.room_identifier = view_dict['room_identifier']
         if 'light_path' in view_dict and view_dict['light_path'] is not None:
             view.light_path = view_dict['light_path']
+        if 'group_identifier' in view_dict and view_dict['group_identifier'] is not None:
+            view_dict.group_identifier = view_dict['group_identifier']
         return view
 
     @classmethod
@@ -501,7 +558,7 @@ class View(object):
         }
 
         # parse the string here
-        options = parse_radiance_options(view_string)
+        options = cls._parse_radiance_options(view_string)
 
         for opt, value in options.items():
             if opt in mapper:
@@ -536,6 +593,141 @@ class View(object):
             'View file must start with rvu not %s' % view_string[:3]
         return cls.from_string(identifier, view_string)
 
+    @classmethod
+    def from_grid(cls, grid, identifier='from_grid'):
+        """Create view from a grid of views.
+        Generally the grid argument should be the views generated by the grid method.
+        Args:
+            grid: A list of subviews. If only a single view is given, this view will be
+                returned. The views can be either class instances of View, strings or
+                .unf files. If strings are used, the views will be created by the
+                from_string method. If .unf files are used, the views will be created by
+                the from_string method using the view found in the Radiance header.
+            identifier: Text string for a unique View ID. Must not contain spaces
+                or special characters. This will be used to identify the object across
+                a model and in the exported Radiance files. If None, this will be set
+                to 'from_grid'. (Default: 'from_grid')
+        """
+        if not isinstance(grid, (list, tuple)):
+            grid = [grid]
+
+        views = []
+        # check if grid argument views are valid
+        for c, view in enumerate(grid):
+            if isinstance(view, View):
+                views.append(view)
+            elif view.endswith('.unf'):
+                try:
+                    f = open(view, 'r', encoding='utf-8', errors='ignore')
+                except Exception:
+                    f = open(view, 'r')
+                try:
+                    for line in f:
+                        if not line.strip():
+                            break
+                        else:
+                            low_line = line.strip().lower()
+                            if low_line.startswith('view='):
+                                print(low_line)
+                                views.append(cls.from_string('view_%04d' % c, low_line))
+                except Exception:
+                    raise ValueError('Failed to find view in Radiance header.')
+                finally:
+                    f.close()
+            elif isinstance(view, str):
+                views.append(cls.from_string('view_%04d' % c, view))
+            else:
+                raise ValueError(
+                    'Expected Honeybee Radiance View, string or .unf file.'
+                    'Got: {}'.format(type(view))
+                )
+
+        # if only a single (valid) view is given, then return the view
+        if len(views) == 1:
+            return views[0]
+
+        _type = set()
+        _view_point = set()
+        _view_direction = set()
+        _up_direction = set()
+        _vh = set()
+        _vv = set()
+        _x_div_count = set()
+        _y_div_count = set()
+        # check if type, view point, view direction, and up direction are equal in views
+        # all unique values are collected, all except -vh and -vv must be the same
+        for _view in views:
+            _type.add(_view.type)
+            _view_point.add(_view.position)
+            _view_direction.add(_view.direction)
+            _up_direction.add(_view.up_vector)
+            _vh.add(_view.h_size)
+            _vv.add(_view.v_size)
+            _x_div_count.add(_view.shift)
+            _y_div_count.add(_view.lift)
+
+        if len(_type) > 1:
+            raise ValueError('All subviews must have the same view type.')
+        if len(_view_point) > 1:
+            raise ValueError('All subviews must have the same view point.')
+        if len(_view_direction) > 1:
+            raise ValueError('All subviews must have the same view direction.')
+        if len(_up_direction) > 1:
+            raise ValueError('All subviews must have the same up direction.')
+        if len(_vh) > 1:
+            raise ValueError('All subviews must have the same horizontal view size.')
+        if len(_vv) > 1:
+            raise ValueError('All subviews must have the same vertical view size.')
+
+        # find the grid dimensions (x_count, y_count)
+        x_div_count = len(_x_div_count)
+        y_div_count = len(_y_div_count)
+
+        # get the horizontal and vertical view size from the subviews
+        _vh = list(_vh)[0]
+        _vv = list(_vv)[0]
+
+        # create instance of view
+        view = cls(
+            identifier=identifier,
+            position=list(_view_point)[0],
+            direction=list(_view_direction)[0],
+            up_vector=list(_up_direction)[0],
+            type=list(_type)[0])
+
+        PI = math.pi
+        # find the horizontal and vertical size
+        if view.type == 'l' or view.type == 'a':
+            # parallel view (vtl) or angular fisheye (vta)
+            h_size = _vh * x_div_count
+            v_size = _vv * y_div_count
+
+        elif view.type == 'v':
+            # perspective (vtv)
+            pi2 = (2. * 180. / PI)
+            h_size = pi2 * math.tan(_vh / (2. * 180. / PI)) * x_div_count
+            v_size = pi2 * math.atan(math.tan(_vv / (2. * 180. / PI)) * y_div_count)
+
+        elif view.type == 'h':
+            # hemispherical fisheye (vth)
+            pi2 = (2. * 180. / PI)
+            h_size = pi2 * math.asin(math.sin(_vh / (2. * 180. / PI)) * x_div_count)
+            v_size = pi2 * math.asin(math.sin(_vv / (2. * 180. / PI)) * y_div_count)
+
+        else:
+            raise ValueError(
+                'Grid views are not supported for %s.' % view.type)
+
+        # round the number to avoid cases like 59.99999999999999 when should be 60
+        h_size = round(h_size, 10)
+        v_size = round(v_size, 10)
+
+        # update horizontal and vertical view size
+        view.h_size = h_size
+        view.v_size = v_size
+
+        return view
+
     def dimension(self, x_res=None, y_res=None):
         """Get dimensions for this view as '-x %d -y %d [-ld-]'.
 
@@ -543,11 +735,7 @@ class View(object):
         match Radiance defaults.
         """
         x, y = self.dimension_x_y(x_res, y_res)
-        return '-x %d -y %d -ld%s' % (
-            x, y,
-            '-' if (self.fore_clip.to_radiance() + self.aft_clip.to_radiance() == '')
-            else '+'
-        )
+        return '-x %d -y %d -ld%s' % (x, y, '-' if (self.vo + self.va == '') else '+')
 
     def dimension_x_y(self, x_res=None, y_res=None):
         """Get dimensions for this view as x, y.
@@ -561,8 +749,8 @@ class View(object):
         if self.is_fisheye:
             return min(x_res, y_res), min(x_res, y_res)
 
-        vh = self.h_size.value
-        vv = self.v_size.value
+        vh = self.h_size
+        vv = self.v_size
 
         if self.type == 'v':
             hv_ratio = math.tan(math.radians(vh) / 2.0) / \
@@ -614,27 +802,31 @@ class View(object):
 
         _views = list(range(x_div_count * y_div_count))
 
-        if self.type == 'l':
-            # parallel view (vtl)
+        if self.type in ('l', 'a', 'c'):
+            # parallel view (vtl) or angular fisheye (vta)
             _vh = self.h_size / x_div_count
             _vv = self.v_size / y_div_count
 
         elif self.type == 'v':
             # perspective (vtv)
-            _vh = (2. * 180. / PI) * \
-                math.atan(((PI / 180. / 2.) * self.h_size) / x_div_count)
-            _vv = (2. * 180. / PI) * \
-                math.atan(math.tan((PI / 180. / 2.) * self.v_size) / y_div_count)
+            pi2 = (2. * 180. / PI)
+            _vh = pi2 * math.atan(((PI / 180. / 2.) * self.h_size) / x_div_count)
+            _vv = pi2 * math.atan(math.tan((PI / 180. / 2.) * self.v_size) / y_div_count)
 
-        elif self.is_fisheye:
-            # fish eye
-            _vh = (2. * 180. / PI) * \
-                math.asin(math.sin((PI / 180. / 2.) * self.h_size) / x_div_count)
-            _vv = (2. * 180. / PI) * \
-                math.asin(math.sin((PI / 180. / 2.) * self.v_size) / y_div_count)
+        elif self.type == 's':
+            # planisphere (stereographic)
+            pi2 = (2. * 180. / PI * 2)
+            _vh = pi2 * math.atan(math.sin((PI / 180. / 2.) * self.h_size) / x_div_count)
+            _vv = pi2 * math.atan(math.sin((PI / 180. / 2.) * self.v_size) / y_div_count)
+
+        elif self.type in ('h'):
+            # hemispherical fish eye
+            pi2 = (2. * 180. / PI)
+            _vh = pi2 * math.asin(math.sin((PI / 180. / 2.) * self.h_size) / x_div_count)
+            _vv = pi2 * math.asin(math.sin((PI / 180. / 2.) * self.v_size) / y_div_count)
 
         else:
-            print("Grid views are not supported for %s." % self.type.to_radiance())
+            print("Grid views are not supported for %s." % self.type)
             return [self]
 
         # create a set of new views
@@ -649,12 +841,12 @@ class View(object):
             if y_div_count == 1:
                 _vl = 0
             else:
-                _vl = ((int(view_count / y_div_count) / (y_div_count - 1)) - 0.5) \
+                _vl = ((int(view_count % y_div_count) / (y_div_count - 1)) - 0.5) \
                     * (y_div_count - 1)
 
-            # create a copy from the current copy
-            _n_view = View('%s_%d' % (self.identifier, view_count))
-
+            # create a copy from the current view
+            _n_view = self.duplicate()
+            _n_view.identifier = '%s_%d' % (self.identifier, view_count)
             # update parameters
             _n_view.h_size = _vh
             _n_view.v_size = _vv
@@ -662,8 +854,6 @@ class View(object):
             _n_view.lift = _vl
             _n_view._fore_clip = self._fore_clip
             _n_view._aft_clip = self._aft_clip
-            _n_view._room_identifier = self._room_identifier
-            _n_view._light_path = self._light_path
             try:
                 _n_view.display_name = '%s_%d' % (self.display_name, view_count)
             except UnicodeEncodeError:  # character no found on machine
@@ -689,7 +879,7 @@ class View(object):
         """Get a dictionary with information about the View.
 
         This can be written as a JSON into a model radiance folder to narrow
-        down the number of aperture groups that have to be run with this sensor grid.
+        down the number of aperture groups that have to be run with this view.
 
         Args:
             model: A honeybee Model object which will be used to identify
@@ -700,6 +890,10 @@ class View(object):
             base['light_path'] = self._light_path
         elif model and self._room_identifier:  # auto-calculate the light path
             base['light_path'] = light_path_from_room(model, self._room_identifier)
+
+        if self._group_identifier:
+            base['group_identifier'] = self._group_identifier
+
         return base
 
     def to_dict(self):
@@ -707,16 +901,16 @@ class View(object):
         base = {
             'type': 'View',
             'identifier': self.identifier,
-            'position': self.position.value,
-            'direction': self.direction.value,
-            'up_vector': self.up_vector.value,
-            'h_size': self.h_size.value,
-            'v_size': self.v_size.value,
-            'shift': self.shift.value,
-            'lift': self.lift.value,
-            'view_type': self.type.value,
-            'fore_clip': self.fore_clip.value,
-            'aft_clip': self.aft_clip.value
+            'position': self.position,
+            'direction': self.direction,
+            'up_vector': self.up_vector,
+            'h_size': self.h_size,
+            'v_size': self.v_size,
+            'shift': self.shift,
+            'lift': self.lift,
+            'view_type': self.type,
+            'fore_clip': self.fore_clip,
+            'aft_clip': self.aft_clip
         }
         if self._display_name is not None:
             base['display_name'] = self.display_name
@@ -724,6 +918,8 @@ class View(object):
             base['room_identifier'] = self.room_identifier
         if self._light_path is not None:
             base['light_path'] = self.light_path
+        if self._group_identifier is not None:
+            base['group_identifier'] = self.group_identifier
         return base
 
     def to_file(self, folder, file_name=None, mkdir=False):
@@ -740,6 +936,8 @@ class View(object):
         """
 
         identifier = file_name or self.identifier + '.vf'
+        if not (identifier.endswith('.vf') or identifier.endswith('.unf')):
+            identifier += '.vf'
         # add rvu before the view itself
         content = 'rvu ' + self.to_radiance()
         return futil.write_to_file_by_name(folder, identifier, content, mkdir)
@@ -829,6 +1027,41 @@ class View(object):
         self.direction = plane.x * view_direction.magnitude
         self.up_vector = plane.n * view_up_vector.magnitude
 
+    @staticmethod
+    def _parse_radiance_options(string):
+        """Parse a radiance option string (e.g. '-ab 4 -ad 256').
+
+        The string should start with a '-' otherwise it will be trimmed to the
+        first '-' in the string.
+        """
+        try:
+            index = string.index('-')
+        except ValueError:
+            if not ' '.join(string.split()).replace('"', '').replace("'", '').strip():
+                return {}
+            raise ValueError(
+                'Invalid Radiance options string input. '
+                'Failed to find - in input string.'
+            )
+
+        _rad_opt_pattern = r'-[a-zA-Z]+'
+        _rad_opt_compiled_pattern = re.compile(_rad_opt_pattern)
+        sub_string = ' '.join(string[index:].split())
+        value = re.split(_rad_opt_compiled_pattern, sub_string)[1:]
+        key = re.findall(_rad_opt_pattern, sub_string)
+
+        options = collections.OrderedDict()
+        for k, v in zip(key, value):
+            values = v.split()
+            count = len(values)
+            if count == 0:
+                values = ''
+            elif count == 1:
+                values = values[0]
+            options[k[1:]] = values
+
+        return options
+
     def duplicate(self):
         """Get a copy of this object."""
         return self.__copy__()
@@ -849,9 +1082,9 @@ class View(object):
 
     def __key(self):
         """A tuple based on the object properties, useful for hashing."""
-        return (self.identifier, hash(self.position.value), hash(self.direction.value),
-                hash(self.up_vector.value), self.type.value, self.h_size.value,
-                self.v_size.value, self.shift.value, self.lift.value, self._display_name,
+        return (self.identifier, hash(self.position), hash(self.direction),
+                hash(self.up_vector), self.type, self.h_size,
+                self.v_size, self.shift, self.lift, self._display_name,
                 self._room_identifier)
 
     def __hash__(self):
