@@ -1,5 +1,6 @@
 # coding=utf-8
 """Methods to write to rad."""
+from honeybee_radiance.sensorgrid import SensorGrid
 from ladybug.futil import write_to_file_by_name, preparedir
 from honeybee.config import folders
 from honeybee.boundarycondition import Surface
@@ -7,12 +8,16 @@ from honeybee_radiance_folder.folder import ModelFolder
 import honeybee_radiance_folder.config as folder_config
 
 from .geometry import Polygon
-from .modifier.material import BSDF
+from .modifier.material import aBSDF, BSDF
 from .lib.modifiers import black
 
 import os
+import sys
 import json
 import shutil
+import re
+import itertools
+from collections import defaultdict
 
 
 def shade_to_rad(shade, blk=False, minimal=False):
@@ -83,7 +88,7 @@ def aperture_to_rad(aperture, blk=False, minimal=False):
     return '\n\n'.join(ap_strs)
 
 
-def face_to_rad(face, blk=False, minimal=False):
+def face_to_rad(face, blk=False, minimal=False, exclude_sub_faces=False):
     """Get Face as a Radiance string.
 
     Note that the resulting string does not include modifier definitions. Nor
@@ -97,7 +102,9 @@ def face_to_rad(face, blk=False, minimal=False):
             be output, which is useful for direct studies and isolation studies
             to understand the contribution of individual apertures.
         minimal: Boolean to note whether the radiance string should be written
-            in a minimal format (with spaces instead of line breaks). Default: False.
+            in a minimal format (with spaces instead of line breaks). (Default: False).
+        exclude_sub_faces:Boolean to note whether Apertures and Doors should
+            be excluded from the output string. (Default: False).
     """
     rad_prop = face.properties.radiance
     modifier = rad_prop.modifier_blk if blk else rad_prop.modifier
@@ -105,10 +112,11 @@ def face_to_rad(face, blk=False, minimal=False):
     face_strs = [rad_poly.to_radiance(minimal, False, False)]
     for shd in face.shades:
         face_strs.append(shade_to_rad(shd, blk, minimal))
-    for dr in face.doors:
-        face_strs.append(door_to_rad(dr, blk, minimal))
-    for ap in face.apertures:
-        face_strs.append(aperture_to_rad(ap, blk, minimal))
+    if not exclude_sub_faces:
+        for dr in face.doors:
+            face_strs.append(door_to_rad(dr, blk, minimal))
+        for ap in face.apertures:
+            face_strs.append(aperture_to_rad(ap, blk, minimal))
     return '\n\n'.join(face_strs)
 
 
@@ -178,15 +186,20 @@ def model_to_rad(model, blk=False, minimal=False):
     # write all Faces into the file
     model_str = ['#   ================ MODEL ================\n']
     faces = model.faces
-    interior_faces = set()
+    interior_faces, offset = set(), model.tolerance * -2
     if len(faces) != 0:
         model_str.append('#   ================ FACES ================\n')
         for face in faces:
             if isinstance(face.boundary_condition, Surface):
                 if face.identifier in interior_faces:
-                    continue
-                interior_faces.add(face.boundary_condition.boundary_condition_object)
-            model_str.append(face_to_rad(face, blk, minimal))
+                    face = face.duplicate()
+                    face.move(face.normal * offset)
+                    model_str.append(face_to_rad(face, blk, minimal, True))
+                else:
+                    interior_faces.add(face.boundary_condition.boundary_condition_object)
+                    model_str.append(face_to_rad(face, blk, minimal))
+            else:
+                model_str.append(face_to_rad(face, blk, minimal))
 
     # write all orphaned Apertures into the file
     apertures = model.orphaned_apertures
@@ -230,7 +243,10 @@ def model_to_rad(model, blk=False, minimal=False):
     return '\n\n'.join(model_str), '\n\n'.join(modifier_str)
 
 
-def model_to_rad_folder(model, folder=None, config_file=None, minimal=False):
+def model_to_rad_folder(
+    model, folder=None, config_file=None, minimal=False, grids=None, views=None,
+    full_match=False
+):
     r"""Write a honeybee model to a rad folder.
 
     The rad files in the resulting folders will include all geometry (Rooms, Faces,
@@ -249,11 +265,22 @@ def model_to_rad_folder(model, folder=None, config_file=None, minimal=False):
             will be used. (Default: None).
         minimal: Boolean to note whether the radiance strings should be written
             in a minimal format (with spaces instead of line breaks). Default: False.
+        grids: A list of sensor grid names to filter the sensor grids in the
+            model. Use this argument to indicate specific sensor grids that should
+            be included. By default all the sensor grids will be exported. You can use
+            wildcard symbols in names. Use relative path from inside grids folder.
+        views: A list of view files that should be exported to folder. Use this argument
+            to limit what will be written to the radiance folder. You can use wildcard
+            symbols in names. Use relative path from inside views folder.
+        full_match: A boolean to filter grids and views by their identifiers as full
+            matches. Setting this to True indicates that wildcard symbols will not be
+            used in the filtering of grids and views. In this case the names of grids
+            and views are filtered as is. (Default: False).
     """
     # prepare the folder for simulation
     model_id = model.identifier
     if folder is None:
-        folder = os.path.join(folders.default_simulation_folder, model_id, 'Radiance')
+        folder = os.path.join(folders.default_simulation_folder, model_id, 'radiance')
     if not os.path.isdir(folder):
         preparedir(folder)  # create the directory if it's not there
     model_folder = ModelFolder(folder, 'model', config_file)
@@ -296,7 +323,9 @@ def model_to_rad_folder(model, folder=None, config_file=None, minimal=False):
                 st_d = _write_dynamic_subface_files(
                     folder, out_subfolder, group, minimal)
                 _write_mtx_files(folder, out_subfolder, group, st_d, minimal)
+
                 ext_dict[group.identifier] = st_d
+
         _write_dynamic_json(folder, out_subfolder, ext_dict)
 
     # write dynamic shade groups
@@ -327,62 +356,150 @@ def model_to_rad_folder(model, folder=None, config_file=None, minimal=False):
     bsdf_mods = model.properties.radiance.bsdf_modifiers
     if len(bsdf_mods) != 0:
         preparedir(bsdf_folder)
+        bsdfs_info = []
         for bdf_mod in bsdf_mods:
             bsdf_name = os.path.split(bdf_mod.bsdf_file)[-1]
             new_bsdf_path = os.path.join(bsdf_folder, bsdf_name)
             shutil.copy(bdf_mod.bsdf_file, new_bsdf_path)
+            bsdfs_info.append(
+                {
+                    'name': bdf_mod.display_name,
+                    'identifier': bdf_mod.identifier,
+                    'path': os.path.join(model_folder.bsdf_folder(full=False), bsdf_name)
+                })
+        bsdf_info_file = os.path.join(bsdf_folder, '_info.json')
+        with open(bsdf_info_file, 'w') as fp:
+            json.dump(bsdfs_info, fp, indent=2)
 
     # write the assigned sensor grids and views into the correct folder
     grid_dir = model_folder.grid_folder(full=True)
-    grids = model.properties.radiance.sensor_grids
-    if len(grids) != 0:
+    _write_sensor_grids(grid_dir, model, grids, full_match)
+    view_dir = model_folder.view_folder(full=True)
+    _write_views(view_dir, model, views, full_match)
+
+    model_folder.combined_receivers(auto_mtx_path=False)
+    return folder
+
+
+def _write_sensor_grids(folder, model, grids_filter, full_match=False):
+    """Write out the sensor grid files.
+
+    Args:
+        folder: The sensor grids folder.
+        model: A Honeybee model.
+        grids_filter: A list of sensor grid names to filter the sensor grids in the
+            model. Use this argument to indicate specific sensor grids that should
+            be included. By default all the sensor grids will be exported. You can use
+            wildcard symbols in names. Use relative path from inside grids folder.
+        full_match: A boolean to filter grids by their identifiers as full matches.
+            (Default: False).
+
+    Returns:
+        A tuple for path to _info.json and _model_grids_info.json. The first file
+        includes the information for the sensor grids that are written to the folder and
+        the second one is the information for the input sensor grids from the model.
+
+        Use ``_info.json`` for access the sensor grid information for running the
+        commands and ``_model_grids_info`` file for loading the results back to match
+        with the model. Model_grids_info has an extra key for `start_ln` which provides
+        the start line for where the sensors for this grid starts in a pts file. Unless
+        there are grids with same identifier this value will be set to 0.
+
+    """
+    sensor_grids = model.properties.radiance.sensor_grids
+    filtered_grids = _filter_by_pattern(
+        sensor_grids, grids_filter, full_match=full_match)
+    if len(filtered_grids) != 0:
         grids_info = []
-        preparedir(grid_dir)
-        for grid in grids:
-            fp = grid.to_file(grid_dir)
-            info_dir = os.path.dirname(fp)
-            info_file = os.path.join(info_dir, '{}.json'.format(grid.identifier))
-            with open(info_file, 'w') as fp:
-                json.dump(grid.info_dict(model), fp, indent=4)
-
-            grid_info = {
-                'name': grid.identifier,
-                'identifier': grid.identifier,
-                'count': grid.count,
-                'group': grid.group_identifier or ''
-            }
-
-            grids_info.append(grid_info)
+        preparedir(folder)
+        # group_by_identifier
+        grouped_grids = _group_by_identifier(filtered_grids)
+        for grid in grouped_grids:
+            grid.to_file(folder)
+            grids_info.append(grid.info_dict(model))
 
         # write information file for all the grids.
-        grids_info_file = os.path.join(grid_dir, '_info.json')
-        with open(grids_info_file, 'w') as fp:
-            json.dump(grids_info, fp, indent=2)
+        grids_info_file = os.path.join(folder, '_info.json')
+        if (sys.version_info < (3, 0)):  # we need to manually encode it as UTF-8
+            with open(grids_info_file, 'wb') as fp:
+                info_str = json.dumps(grids_info, indent=2, ensure_ascii=False)
+                fp.write(info_str.encode('utf-8'))
+        else:
+            with open(grids_info_file, 'w', encoding='utf-8') as fp:
+                json.dump(grids_info, fp, indent=2, ensure_ascii=False)
 
-    view_dir = model_folder.view_folder(full=True)
-    views = model.properties.radiance.views
-    if len(views) != 0:
+        # write input grids info
+        model_grids_info = []
+        start_line = defaultdict(lambda: 0)
+        for grid in filtered_grids:
+            identifier = grid.identifier
+            grid_info = {
+                'name': identifier,
+                'identifier': identifier,
+                'count': grid.count,
+                'group': grid.group_identifier or '',
+                'full_id': grid.full_identifier,
+                'start_ln': start_line[identifier],
+            }
+
+            start_line[identifier] += grid.count
+
+            model_grids_info.append(grid_info)
+
+        model_grids_info_file = os.path.join(folder, '_model_grids_info.json')
+        with open(model_grids_info_file, 'w') as fp:
+            json.dump(model_grids_info, fp, indent=2)
+
+        return grids_info_file, model_grids_info_file
+    elif len(sensor_grids) != 0:
+        raise ValueError('All sensor grids were filtered out of the model folder!')
+
+
+def _write_views(folder, model, views_filter, full_match=False):
+    """Write out the view files.
+
+    Args:
+        folder: The views folder.
+        model: A Honeybee model.
+        views_filter: A list of view names to filter the views in the model. Use this
+            argument to indicate specific views that should be included. By default,
+            all the views will be exported. You can use wildcard symbols in names.
+            Use relative path from inside views folder.
+        full_match: A boolean to filter views by their identifiers as full matches.
+            (Default: False).
+
+    Returns:
+        The path to _info.json, which includes the information for the views that
+        are written to the folder.
+    """
+    model_views = model.properties.radiance.views
+    filtered_views = _filter_by_pattern(model_views, views_filter, full_match=full_match)
+    if len(filtered_views) != 0:
+        preparedir(folder)
+        # group_by_identifier
         views_info = []
-        preparedir(view_dir)
-        for view in views:
-            view.to_file(view_dir)
-            info_file = os.path.join(view_dir, '{}.json'.format(view.identifier))
+        for view in filtered_views:
+            view.to_file(folder)
+            info_file = os.path.join(folder, '{}.json'.format(view.identifier))
             with open(info_file, 'w') as fp:
                 json.dump(view.info_dict(model), fp, indent=4)
 
-            # TODO: see if it make sense to use to_dict here instead of only taking the
-            # name
             view_info = {
-                'name': view.identifier
+                'name': view.identifier,
+                'identifier': view.identifier,
+                'group': view.group_identifier or '',
+                'full_id': view.full_identifier
             }
             views_info.append(view_info)
 
         # write information file for all the views.
-        views_info_file = os.path.join(view_dir, '_info.json')
+        views_info_file = os.path.join(folder, '_info.json')
         with open(views_info_file, 'w') as fp:
             json.dump(views_info, fp, indent=2)
 
-    return folder
+        return views_info_file
+    elif len(model_views) != 0:
+        raise ValueError('All views were filtered out of the model folder!')
 
 
 def _write_dynamic_shade_files(folder, sub_folder, group, minimal=False):
@@ -488,7 +605,7 @@ def _write_mtx_files(folder, sub_folder, group, states_json_list, minimal=False)
 
     # write the single mtx file if everything is default
     if one_mtx and tmxt_valid:
-        mtx_str = group.vmtx_to_radiance(0, minimal)
+        mtx_str = group.vmtx_to_radiance(state_i, minimal)
         write_to_file_by_name(dest, mtx_file, mtx_str)
 
 
@@ -534,11 +651,15 @@ def _write_static_files(
         if punched_verts:
             for face in geometry:
                 modifier = face.properties.radiance.modifier
-                rad_poly = Polygon(face.identifier, face.punched_vertices, modifier)
+                geo = face.punched_vertices if hasattr(face, 'punched_vertices') \
+                    else face.vertices
+                rad_poly = Polygon(face.identifier, geo, modifier)
                 face_strs.append(rad_poly.to_radiance(minimal, False, False))
             for face, mod_name in zip(geometry_blk, mod_names):
                 modifier = mod_combs[mod_name][0]
-                rad_poly = Polygon(face.identifier, face.punched_vertices, modifier)
+                geo = face.punched_vertices if hasattr(face, 'punched_vertices') \
+                    else face.vertices
+                rad_poly = Polygon(face.identifier, geo, modifier)
                 face_strs.append(rad_poly.to_radiance(minimal, False, False))
         else:
             for face in geometry:
@@ -554,13 +675,13 @@ def _write_static_files(
         mod_strs = []
         mod_blk_strs = []
         for mod in modifiers:
-            if isinstance(mod, BSDF):
+            if isinstance(mod, (aBSDF, BSDF)):
                 _process_bsdf_modifier(mod, mod_strs, minimal)
             else:
                 mod_strs.append(mod.to_radiance(minimal))
         for mod in modifiers_blk:
-            if isinstance(mod, BSDF):
-                _process_bsdf_modifier(mod, mod_strs, minimal)
+            if isinstance(mod, (aBSDF, BSDF)):
+                _process_bsdf_modifier(mod, mod_blk_strs, minimal)
             else:
                 mod_blk_strs.append(mod.to_radiance(minimal))
 
@@ -603,7 +724,7 @@ def _unique_modifier_blk_combinations(geometry_objects):
         -   modifier_combs: A dictionary of modifiers with identifiers of
             modifiers as keys and tuples with two modifiers as values. Each
             item in the dictionary represents a unique combination of modifier
-            and modifer_blk found in the objects. Both modifiers in the pair
+            and modifier_blk found in the objects. Both modifiers in the pair
             have the same identifier (making them write-able into a radiance
             folder). The first item in the tuple is the true modifier while
             the second one is the modifier_blk.
@@ -669,3 +790,64 @@ def _instance_in_array(object_instance, object_array):
         if val is object_instance:
             return True
     return False
+
+
+def _filter_by_pattern(input_objects, filter, full_match=False):
+    """Filter model grids and views based on user input."""
+    if not filter or filter == '*':
+        return input_objects
+
+    if not isinstance(filter, (list, tuple)):
+        filter = [filter]
+    if not full_match:
+        patterns = [
+            re.compile(f.replace('*', '.+').replace('?', '.')) for f in filter
+        ]
+    else:
+        patterns = [
+            re.compile(f) if f.startswith('^') and f.endswith('$') else
+            re.compile('^%s$' % f) for f in filter
+        ]
+    indexes = []
+
+    for count, obj in enumerate(input_objects):
+        try:
+            id_ = obj.full_identifier
+        except AttributeError:
+            id_ = obj['full_id']
+        for pattern in patterns:
+            if re.search(pattern, id_):
+                indexes.append(count)
+    indexes = list(set(indexes))
+    indexes.sort()
+
+    return [input_objects[i] for i in indexes]
+
+
+def _group_by_identifier(sensor_grids):
+    """Group sensor grids or views if they have the same full identifier."""
+    group_func = lambda grid: grid.full_identifier  # noqa: E731
+
+    ordered_sensor_grids = sorted(sensor_grids, key=group_func)
+
+    # check if there is any duplicated identifiers
+    ids = {grid.full_identifier for grid in sensor_grids}
+    if len(list(ids)) == len(sensor_grids):
+        # there is no duplicated identifier - return the original list
+        return sensor_grids
+
+    updated_grids = []
+    for group_identifier, grids in itertools.groupby(ordered_sensor_grids, group_func):
+        grids = list(grids)
+        if len(grids) > 1:
+            # merge grids into one
+            sensors = []
+            for grid in grids:
+                sensors.extend(grid.sensors)
+            joined_grid = SensorGrid(grids[0].identifier, sensors)
+            joined_grid.group_identifier = grids[0].group_identifier
+            updated_grids.append(joined_grid)
+        else:
+            updated_grids.append(grids[0])
+
+    return updated_grids
